@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -73,6 +74,12 @@ MS2_PUBLIC void ms_dtls_srtp_start(MSDtlsSrtpContext* context);
  */
 MS2_PUBLIC void ms_dtls_srtp_context_destroy(MSDtlsSrtpContext *ctx);
 
+/**
+ * Reset DTLS context to renegotiate keys and carry out once again SSL handshake next time DTLS starts
+ * @param[in/out]	context		the DTLS-SRTP context
+ */
+MS2_PUBLIC void ms_dtls_srtp_reset_context(MSDtlsSrtpContext *context);
+
 /**
  * Set DTLS role: server or client, called when SDP exchange reach the point where we can determine self role
  * @param[in/out]	context		the DTLS-SRTP context
@@ -80,6 +87,14 @@ MS2_PUBLIC void ms_dtls_srtp_context_destroy(MSDtlsSrtpContext *ctx);
  */
 MS2_PUBLIC void ms_dtls_srtp_set_role(MSDtlsSrtpContext *context, MSDtlsSrtpRole role);
 
+
+/**
+ * Get DTLS role: server or client, called when SDP exchange reach the point where we can determine self role
+ * @param[in/out]	context		the DTLS-SRTP context
+ * @return		role		Client/Server/Invalid/Unset according to SDP INVITE processing
+ */
+MS2_PUBLIC MSDtlsSrtpRole ms_dtls_srtp_get_role(const MSDtlsSrtpContext *context);
+
 /**
  * Give to the DTLS-SRTP context the peer certificate fingerprint extracted from trusted SDP INVITE,
  * it will be compared(case insensitive) with locally computed one after DTLS handshake is completed successfully and peer certicate retrieved
