@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -67,6 +68,13 @@ MS2_PUBLIC void ms_media_player_free(MSMediaPlayer *obj);
  */
 MS2_PUBLIC void * ms_media_player_get_window_id(const MSMediaPlayer *obj);
 
+/**
+ * Set the window ID
+ * @param obj The player
+ * @param window_id the window ID
+ */
+MS2_PUBLIC void ms_media_player_set_window_id(MSMediaPlayer *obj, void* window_id);
+
 /**
  * Set the "End of File" callback
  * @param obj The player
@@ -100,6 +108,12 @@ MS2_PUBLIC bool_t ms_media_player_open(MSMediaPlayer *obj, const char *filepath)
  */
 MS2_PUBLIC void ms_media_player_close(MSMediaPlayer *obj);
 
+/**
+ * Initialize filters and acquire resources, but don't start playing yet
+ * @param obj The player
+ */
+MS2_PUBLIC void ms_media_player_prepare(MSMediaPlayer *obj);
+
 /**
  * Start playback
  * @param obj The player
@@ -164,6 +178,33 @@ MS2_PUBLIC bool_t ms_media_player_matroska_supported(void);
  */
 MS2_PUBLIC MSFileFormat ms_media_player_get_file_format(const MSMediaPlayer *obj);
 
+/**
+ * Returns whether the file has video and if it can be displayed
+ * @param obj The player
+ * @return TRUE if file has video and it can be displayed, FALSE otherwise
+ */
+MS2_PUBLIC bool_t ms_media_player_get_is_video_available(MSMediaPlayer *obj);
+
+/**
+ * Set the volume gain of the player.
+ * @param player #LinphonePlayer object @notnil
+ * @param gain Percentage of the gain. Valid values are in [ 0.0 : 1.0 ].
+ */
+MS2_PUBLIC void ms_media_player_set_volume_gain(MSMediaPlayer *player, float volume);
+
+/**
+ * Get the volume gain of the player.
+ * @param player #LinphonePlayer object @notnil
+ * @return Percentage of the gain. Valid values are in [ 0.0 : 1.0 ].
+ */
+MS2_PUBLIC float ms_media_player_get_volume_gain(const MSMediaPlayer *player);
+
+/**
+ * Returns a pointer to the video display filter
+ * @param obj The player
+ */
+MS2_PUBLIC MSFilter *ms_media_player_get_video_sink(const MSMediaPlayer *const obj);
+
 #ifdef __cplusplus
 }
 #endif
