@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -40,6 +41,7 @@ struct _MSSndCardManager{
 	MSFactory* factory;
 	MSList *cards;
 	MSList *descs;
+	char *paramString;
 };
 
 /**
@@ -86,6 +88,7 @@ typedef enum _MSSndCardControlElem MSSndCardControlElem;
 struct _MSSndCard;
 
 typedef void (*MSSndCardDetectFunc)(MSSndCardManager *obj);
+typedef bool_t (*MSSndCardReloadRequestedFunc)(MSSndCardManager *obj);
 typedef void (*MSSndCardInitFunc)(struct _MSSndCard *obj);
 typedef void (*MSSndCardUninitFunc)(struct _MSSndCard *obj);
 typedef void (*MSSndCardSetLevelFunc)(struct _MSSndCard *obj, MSSndCardMixerElem e, int percent);
@@ -101,6 +104,7 @@ typedef void (*MSSndCardUnloadFunc)(MSSndCardManager *obj);
 typedef void (*MSSndCardAudioSessionFunc)(struct _MSSndCard *obj, bool_t actived);
 typedef void (*MSSndCardCallKitFunc)(struct _MSSndCard *obj, bool_t enabled);
 typedef void (*MSSndCardAudioRouteFunc)(struct _MSSndCard *obj);
+typedef void (*MSSndCardConfigureFunc)(struct _MSSndCard *obj);
 
 
 struct _MSSndCardDesc{
@@ -121,6 +125,8 @@ struct _MSSndCardDesc{
 	MSSndCardAudioSessionFunc audio_session_activated;
 	MSSndCardCallKitFunc callkit_enabled;
 	MSSndCardAudioRouteFunc audio_route_changed;
+	MSSndCardConfigureFunc configure;
+	MSSndCardReloadRequestedFunc reload_requested;
 };
 
 /**
@@ -147,6 +153,27 @@ enum _MSSndCardStreamType{
  */
 typedef enum _MSSndCardStreamType MSSndCardStreamType;
 
+enum _MSSndCardDeviceType {
+	MS_SND_CARD_DEVICE_TYPE_TELEPHONY,
+	MS_SND_CARD_DEVICE_TYPE_AUX_LINE,
+	MS_SND_CARD_DEVICE_TYPE_GENERIC_USB,
+	MS_SND_CARD_DEVICE_TYPE_HEADSET,
+	MS_SND_CARD_DEVICE_TYPE_MICROPHONE,
+	MS_SND_CARD_DEVICE_TYPE_EARPIECE,
+	MS_SND_CARD_DEVICE_TYPE_HEADPHONES,
+	MS_SND_CARD_DEVICE_TYPE_SPEAKER,
+	MS_SND_CARD_DEVICE_TYPE_BLUETOOTH,
+	MS_SND_CARD_DEVICE_TYPE_BLUETOOTH_A2DP,
+	MS_SND_CARD_DEVICE_TYPE_UNKNOWN,
+	MS_SND_CARD_DEVICE_TYPE_HEARING_AID
+};
+
+/**
+ * device type enum.
+ * @var DeviceType
+**/
+typedef enum _MSSndCardDeviceType MSSndCardDeviceType;
+
 #define MS_SND_CARD_CAP_DISABLED (0) /**<This soundcard is disabled.*/
 #define MS_SND_CARD_CAP_CAPTURE (1) /**<This sound card can capture sound */
 #define MS_SND_CARD_CAP_PLAYBACK (1<<1) /**<This sound card can playback sound */
@@ -158,11 +185,15 @@ struct _MSSndCard{
 	MSSndCardManager* sndcardmanager;
 	char *name;
 	char *id;
+	int  internal_id;
 	unsigned int capabilities;
+	MSSndCardDeviceType device_type;
 	void *data;
 	int preferred_sample_rate;
 	int latency;
 	MSSndCardStreamType streamType;
+	int ref_count;
+	const SoundDeviceDescription *device_description;
 };
 
 #ifdef __cplusplus
@@ -204,15 +235,51 @@ MS2_PUBLIC MSSndCardManager * ms_snd_card_manager_new(void);
 MS2_PUBLIC void ms_snd_card_manager_destroy(MSSndCardManager* sndcardmanager);
 
 /**
- * Retreive a sound card object based on its name.
+ * Set the sound card manager's parameter string
+ *
+ * @param m    A sound card manager.
+ * @param paramString  A string of the form "param1=true;param2=42;param3=false"
+ *
+ * Returns: MSSndCard if successfull, NULL otherwise.
+ */
+MS2_PUBLIC void ms_snd_card_manager_set_param_string(MSSndCardManager *m, const char *paramString);
+
+/**
+ * Retreive a sound card object based on it's id.
+ * The id can be a regex string. In this case, the search will be done on "Driver_type : ID."
+ * eg: "PulseAudio?( .*)?:.*Build in.*" will retrieve a card with an ID that contains "Build in" and use the PulseAudio Driver.
  *
  * @param m    A sound card manager containing sound cards.
- * @param id   A name for card to search.
+ * @param id   An id for card to search.
  *
  * Returns: MSSndCard if successfull, NULL otherwise.
  */
 MS2_PUBLIC MSSndCard * ms_snd_card_manager_get_card(MSSndCardManager *m, const char *id);
 
+/**
+ * Retreive a sound card object based on it's id and capabilities.
+ * The id can be a regex string. In this case, the search will be done on "Filter : ID."
+ * eg: "PulseAudio?( .*)?:.*Build-in.*" will retrieve a card with an ID that contains "Build-in" in PulseAudio Filter.
+ *
+ * @param m    A sound card manager containing sound cards.
+ * @param id   An id for card to search.
+ * @param capabilities A capabilities mask of MS_SND_CARD_CAP_PLAYBACK and/or MS_SND_CARD_CAP_CAPTURE
+ *
+ * Returns: MSSndCard if successfull, NULL otherwise.
+ */
+MS2_PUBLIC MSSndCard * ms_snd_card_manager_get_card_with_capabilities(MSSndCardManager *m, const char *id, unsigned int capabilities);
+
+/**
+ * Retreive the first sound card object in the card manager based on its type and driver type.
+ *
+ * @param m             A sound card manager containing sound cards.
+ * @param driver_type   The type of the driver.
+ * @param type          The type of the card.
+ *
+ * Returns: MSSndCard if successfull, NULL otherwise.
+ */
+MS2_PUBLIC MSSndCard * ms_snd_card_manager_get_card_by_type(MSSndCardManager *m, const MSSndCardDeviceType type, const char * driver_type);
+
 /**
  * Retreive a playback capable sound card object based on its name.
  *
@@ -223,6 +290,16 @@ MS2_PUBLIC MSSndCard * ms_snd_card_manager_get_card(MSSndCardManager *m, const c
  */
 MS2_PUBLIC MSSndCard * ms_snd_card_manager_get_playback_card(MSSndCardManager *m, const char *id);
 
+/**
+ * Retreive all sound cards having the name provided as input.
+ *
+ * @param m      A sound card manager containing sound cards.
+ * @param name   A name for card to search.
+ *
+ * Returns: MSSndCard list of cards if successfull, NULL otherwise.
+ */
+MS2_PUBLIC bctbx_list_t * ms_snd_card_manager_get_all_cards_with_name(MSSndCardManager *m, const char *name);
+
 /**
  * Retreive a capture capable sound card object based on its name.
  *
@@ -312,12 +389,67 @@ MS2_PUBLIC void ms_snd_card_manager_prepend_cards(MSSndCardManager *m, MSList *l
  */
 MS2_PUBLIC void ms_snd_card_manager_register_desc(MSSndCardManager *m, MSSndCardDesc *desc);
 
+/**
+ * Unregister a sound card description in a sound card manager.
+ *
+ * @param m      A sound card manager containing sound cards.
+ * @param desc   A sound card description object.
+ *
+ */
+MS2_PUBLIC void ms_snd_card_manager_unregister_desc(MSSndCardManager *m, MSSndCardDesc *desc);
+
 /**
  * Ask all registered MSSndCardDesc to re-detect their soundcards.
  * @param m The sound card manager.
 **/
 MS2_PUBLIC void ms_snd_card_manager_reload(MSSndCardManager *m);
 
+/**
+ Check if the manager  reload is requested.
+ Specific for iOS: sometimes auido routes add/remove devices when there are no calls. In this case, the manager reload is requested to update  devices.
+ * @param m   Card Manager
+ * Returns: true if the manager reload is requested.
+ */
+MS2_PUBLIC bool_t ms_snd_card_manager_reload_requested(MSSndCardManager *m);
+
+/**
+ * Check if there is another card in the manager having same driver_type, name and device_type
+ * @param m    Card Manager
+ * @param card Card to compare properties against
+ * @param checkCapabilities flag to check capabilities
+ *
+ * Returns: true if a duplicate has been found, false otherwise
+**/
+MS2_PUBLIC bool_t ms_snd_card_is_card_duplicate(MSSndCardManager *m, MSSndCard *card, bool_t checkCapabilities);
+
+/**
+ * Check if there is another card in the manager having same driver_type, name and device_type
+ * @param m    Card Manager
+ * @param card Card to compare properties against
+ * @param checkCapabilities flag to check capabilities
+ *
+ * Returns: The duplicate if it is found, NULL otherwise
+**/
+MS2_PUBLIC MSSndCard* ms_snd_card_get_card_duplicate(MSSndCardManager *m, MSSndCard *card, bool_t checkCapabilities);
+
+/**
+ * Prevent card type to be at the head fo the list
+ * @param m    Card Manager
+ * @param type Card type to remove from the head of list of cards
+ *
+**/
+MS2_PUBLIC void ms_snd_card_remove_type_from_list_head(MSSndCardManager *m, MSSndCardDeviceType type);
+
+/**
+ * Swap two position of 2 sound cards in the sound card manager.
+ * @param m       Card Manager
+ * @param card0   Card to be swapped
+ * @param card1   Card to be swapped
+ *
+ * Returns: true if card0 and card1 are not null and both are found among the list of sound cards in the card manager, false otherwise
+**/
+MS2_PUBLIC bool_t ms_snd_card_manager_swap_cards(MSSndCardManager *m, MSSndCard *card0, MSSndCard *card1);
+
 /* This function is available for testing only, this should not be used in a real application! */
 MS2_PUBLIC void ms_snd_card_manager_bypass_soundcard_detection(bool_t value);
 
@@ -370,6 +502,7 @@ MS2_PUBLIC MSSndCard * ms_snd_card_new_with_name(MSSndCardDesc *desc,const char*
  * Destroy sound card object.
  *
  * @param obj   A MSSndCard object.
+ * @deprecated, use ms_snd_card_unref instead
  */
 MS2_PUBLIC void ms_snd_card_destroy(MSSndCard *obj);
 
@@ -384,6 +517,16 @@ MS2_PUBLIC void ms_snd_card_destroy(MSSndCard *obj);
  */
 MS2_PUBLIC MSSndCard * ms_snd_card_dup(MSSndCard *card);
 
+/**
+ * Retreive a sound card's device type.
+ *
+ * @param obj   A sound card object.
+ *
+ * Returns: an MSSndCardDeviceType enum type.
+ * Default value is MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_UNKNOWN.
+ */
+MS2_PUBLIC MSSndCardDeviceType ms_snd_card_get_device_type(const MSSndCard *obj);
+
 /**
  * Retreive a sound card's driver type string.
  *
@@ -395,6 +538,15 @@ MS2_PUBLIC MSSndCard * ms_snd_card_dup(MSSndCard *card);
  */
 MS2_PUBLIC const char *ms_snd_card_get_driver_type(const MSSndCard *obj);
 
+/**
+ * Retreive a sound card's device type string.
+ *
+ * @param type   A sound card type.
+ *
+ * Returns: a string if successfull, "bad type" otherwise.
+ */
+MS2_PUBLIC const char * ms_snd_card_device_type_to_string(const MSSndCardDeviceType type);
+
 /**
  * Retreive a sound card's name.
  *
@@ -411,8 +563,22 @@ MS2_PUBLIC const char *ms_snd_card_get_name(const MSSndCard *obj);
  *
  * Returns: A string if successfull, NULL otherwise.
  */
-MS2_PUBLIC const char *ms_snd_card_get_string_id(MSSndCard *obj);
+MS2_PUBLIC const char *ms_snd_card_get_string_id(const MSSndCard *obj);
 
+/**
+ * Retrieve sound card's internal ID.
+ *
+ * @param obj    A sound card object.
+ *
+ * Returns: An integer storing the internal ID value.
+ */
+MS2_PUBLIC int ms_snd_card_get_internal_id(MSSndCard *obj);
+
+/**
+ * Returns TRUE if both MSSndCard are equal, which means that they refer to the same
+ * audio device with same capabilities.
+ */
+MS2_PUBLIC bool_t ms_snd_card_equals(const MSSndCard *c1, const MSSndCard *c2);
 
 /**
  * Retreive sound card's capabilities.
@@ -456,6 +622,15 @@ MS2_PUBLIC int ms_snd_card_get_minimal_latency(MSSndCard *obj);
  */
 MS2_PUBLIC void ms_snd_card_set_level(MSSndCard *obj, MSSndCardMixerElem e, int percent);
 
+/**
+ * Set internal ID of the sound card.
+ *
+ * @param obj      A sound card object.
+ * @param id       A sound card internal ID.
+ *
+ */
+MS2_PUBLIC void ms_snd_card_set_internal_id(MSSndCard *obj, int id);
+
 /**
  * Get some mixer level value.
  *
@@ -577,6 +752,12 @@ MS2_PUBLIC void ms_snd_card_notify_audio_route_changed(MSSndCard *obj);
  */
 MS2_PUBLIC void ms_snd_card_app_notifies_activation(MSSndCard *obj, bool_t yesno);
 
+/**
+ * Used to configure audio session with default settings. Callkit usage.
+ * @param obj      A sound card object.
+ */
+MS2_PUBLIC void ms_snd_card_configure_audio_session(MSSndCard *obj);
+
 /**
  * Sets the stream type for this soundcard, default is VOICE
 **/
@@ -604,6 +785,28 @@ MS2_PUBLIC MSSndCard * ms_alsa_card_new_custom(const char *pcmdev, const char *m
 **/
 MS2_PUBLIC void ms_alsa_card_set_forced_sample_rate(int samplerate);
 
+/**
+ * Returns a string value of the given MSSndCardDeviceType enum
+ */
+MS2_PUBLIC const char* ms_snd_card_device_type_to_string(MSSndCardDeviceType type);
+
+/**
+ * Takes a ref on a MSSndCard
+ */
+MS2_PUBLIC MSSndCard* ms_snd_card_ref(MSSndCard *sndCard);
+
+/**
+ * Removes a ref from a MSSndCard
+ */
+MS2_PUBLIC void ms_snd_card_unref(MSSndCard *sndCard);
+
+#ifdef __ANDROID__
+/**
+ * Sort cards in order to put earpiece and speaker as first devices of every filter.
+**/
+void ms_snd_card_sort(MSSndCardManager *m);
+#endif // __ANDROID__
+
 /** @} */
 
 #ifdef __cplusplus
