@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -144,6 +145,7 @@ MSSndCardDesc msandroid_sound_card_desc = {
 MSSndCard *msandroid_sound_duplicate(MSSndCard *obj){
 	MSSndCard *card=ms_snd_card_new(&msandroid_sound_card_desc);
 	card->name=ms_strdup(obj->name);
+	card->device_description=obj->device_description;
 	return card;
 }
 
@@ -155,6 +157,7 @@ MSSndCard *msandroid_sound_card_new(MSSndCardManager *m) {
 
 	devices = ms_factory_get_devices_info(m->factory);
 	d = ms_devices_info_get_sound_device_description(devices);
+	card->device_description = d;
 	
 	if (d->flags & DEVICE_HAS_BUILTIN_AEC) {
 		card->capabilities |= MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER;
