@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -550,6 +551,11 @@ static OpenSLESInputContext* opensles_input_context_init() {
 static void android_snd_read_init(MSFilter *obj) {
 	OpenSLESInputContext *ictx = opensles_input_context_init();
 	obj->data = ictx;
+
+	bool permissionGranted = ms_android_is_record_audio_permission_granted();
+	if (!permissionGranted) {
+		ms_error("[OpenSLES] RECORD_AUDIO permission hasn't been granted!");
+	}
 }
 
 static void android_snd_read_preprocess(MSFilter *obj) {
@@ -1047,7 +1053,7 @@ static int android_snd_write_configure_soundcard(MSFilter *obj, void *data) {
 		octx->soundCard = ms_snd_card_ref(card);
 		octx->setContext((OpenSLESContext*)card->data);
 		JNIEnv *env = ms_get_jni_env();
-		ms_android_change_device(env, card->device_type);
+		ms_android_change_device(env, card->internal_id, card->device_type);
 		ms_mutex_unlock(&octx->mutex);
 	}
 	return 0;
@@ -1088,7 +1094,7 @@ static void android_snd_write_preprocess(MSFilter *obj) {
 
 	// Ensure consistency between the soundcard the core is setting and the one actually used as an output
 	JNIEnv *env = ms_get_jni_env();
-	ms_android_change_device(env, octx->soundCard->device_type);
+	ms_android_change_device(env, octx->soundCard->internal_id, octx->soundCard->device_type);
 }
 
 static void android_snd_write_process(MSFilter *obj) {
@@ -1131,7 +1137,7 @@ static void android_snd_write_postprocess(MSFilter *obj) {
 
 	// At the end of a call, postprocess is called therefore here the output device can be changed to earpiece in the audio manager
 	JNIEnv *env = ms_get_jni_env();
-	ms_android_change_device(env, MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE);
+	ms_android_change_device(env, -1, MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE);
 	free(octx->playBuffer[0]);
 	octx->playBuffer[0]=NULL;
 	free(octx->playBuffer[1]);
@@ -1204,7 +1210,7 @@ static void snd_card_device_create_extra_fields(MSSndCardManager *m, MSSndCard *
 	}
 
 	card->latency = deviceDescription->delay;
-	if (deviceDescription->recommended_rate){
+	if (deviceDescription->recommended_rate) {
 		card_data->samplerate = deviceDescription->recommended_rate;
 	}
 }
@@ -1217,6 +1223,7 @@ static void snd_card_device_create(int device_id, const char * name, MSSndCardDe
 	card->name = ms_strdup(name);
 	card->internal_id = device_id;
 	card->device_type = type;
+	card->device_description = deviceDescription;
 
 	// Card capabilities
 	card->capabilities = capabilities;
