@@ -1,24 +1,42 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * androidsound_utils.cpp - Android Audio Utils.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <mediastreamer2/android_utils.h>
 
+bool ms_android_is_record_audio_permission_granted() {
+	JNIEnv *env = ms_get_jni_env();
+	jclass mediastreamerAndroidContextClass = env->FindClass("org/linphone/mediastream/MediastreamerAndroidContext");
+	if (mediastreamerAndroidContextClass != NULL) {
+		jmethodID isPermissionGranted = env->GetStaticMethodID(mediastreamerAndroidContextClass, "isRecordAudioPermissionGranted", "()Z");
+		if (isPermissionGranted != NULL) {
+			jboolean ret = env->CallStaticBooleanMethod(mediastreamerAndroidContextClass, isPermissionGranted);
+			ms_message("[Android Audio Utils] is RECORD_AUDIO permission granted? %i", ret);
+			env->DeleteLocalRef(mediastreamerAndroidContextClass);
+			return (bool) ret;
+		}
+		env->DeleteLocalRef(mediastreamerAndroidContextClass);
+	}
+	ms_error("[Android Audio Utils] Failed to retrive RECORD_AUDIO permission state from MediastreamerAndroidContext!");
+	return true;
+}
+
 int ms_android_get_preferred_buffer_size() {
 	int DevicePreferredBufferSize = -1;
 	JNIEnv *env = ms_get_jni_env();
@@ -128,34 +146,60 @@ bool ms_android_is_audio_route_changes_disabled(JNIEnv *env) {
 	return disabled;
 }
 
-void ms_android_change_device(JNIEnv *env, MSSndCardDeviceType type) {
+void ms_android_change_device(JNIEnv *env, int deviceID, MSSndCardDeviceType type) {
 	if (ms_android_is_audio_route_changes_disabled(env)) return;
 
-	std::string methodName;
-	if (type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_SPEAKER) {
-		methodName = "enableSpeaker";
-	} else if (type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_BLUETOOTH) {
-		methodName = "startBluetooth";
-	} else if (type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE || 
-			type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADSET || 
-			type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADPHONES) 
-	{
-		methodName = "enableEarpiece";
-	}
+	jclass mediastreamerAndroidContextClass = env->FindClass("org/linphone/mediastream/MediastreamerAndroidContext");
+	if (mediastreamerAndroidContextClass != NULL) {
+		bool changeDone = false;
+		if (ms_android_get_sdk_version(env) >= 31) {
+			if (deviceID == -1) {
+				jmethodID clearDevice = env->GetStaticMethodID(mediastreamerAndroidContextClass, "clearCommunicationDevice", "()V");
+				if (clearDevice != NULL) {
+					env->CallStaticVoidMethod(mediastreamerAndroidContextClass, clearDevice);
+					ms_message("[Android Audio Utils] Communication device cleared");
+					changeDone = true;
+				}
+			} else {
+				jmethodID changeDevice = env->GetStaticMethodID(mediastreamerAndroidContextClass, "setCommunicationDevice", "(I)Z");
+				if (changeDevice != NULL) {
+					bool result = env->CallStaticBooleanMethod(mediastreamerAndroidContextClass, changeDevice, deviceID);
+					if (result) {
+						ms_message("[Android Audio Utils] Communication device changed to ID: %i (%s)", deviceID, ms_snd_card_device_type_to_string(type));
+						changeDone = true;
+					} else {
+						ms_error("[Android Audio Utils] Failed to change communication device to ID: %i (%s)", deviceID, ms_snd_card_device_type_to_string(type));
+					}
+				}
+			}
+		}
 
-	if (methodName.empty()) {
-		ms_error("[Android Audio Utils] Unable to find method to enable device type %s", ms_snd_card_device_type_to_string(type));
-	} else {
-		jclass mediastreamerAndroidContextClass = env->FindClass("org/linphone/mediastream/MediastreamerAndroidContext");
-		if (mediastreamerAndroidContextClass != NULL) {
-			jmethodID changeDevice = env->GetStaticMethodID(mediastreamerAndroidContextClass, methodName.c_str(), "()V");
-			if (changeDevice != NULL) {
-					env->CallStaticVoidMethod(mediastreamerAndroidContextClass, changeDevice);
-					ms_message("[Android Audio Utils] changing device to %s ", ms_snd_card_device_type_to_string(type));
+		if (!changeDone) {
+			std::string methodName;
+			if (type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_SPEAKER) {
+				methodName = "enableSpeaker";
+			} else if (type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_BLUETOOTH ||
+					type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEARING_AID) {
+				methodName = "startBluetooth";
+			} else if (type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE || 
+					type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADSET || 
+					type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADPHONES) 
+			{
+				methodName = "enableEarpiece";
+			}
+
+			if (methodName.empty()) {
+				ms_error("[Android Audio Utils] Unable to find method to enable device type %s", ms_snd_card_device_type_to_string(type));
+			} else {
+				jmethodID changeDevice = env->GetStaticMethodID(mediastreamerAndroidContextClass, methodName.c_str(), "()V");
+				if (changeDevice != NULL) {
+						env->CallStaticVoidMethod(mediastreamerAndroidContextClass, changeDevice);
+						ms_message("[Android Audio Utils] changing device to %s ", ms_snd_card_device_type_to_string(type));
+				}
 			}
-			env->DeleteLocalRef(mediastreamerAndroidContextClass);
 		}
 	}
+	env->DeleteLocalRef(mediastreamerAndroidContextClass);
 }
 
 void ms_android_set_bt_enable(JNIEnv *env, const bool_t enable) {
@@ -176,7 +220,7 @@ void ms_android_set_bt_enable(JNIEnv *env, const bool_t enable) {
 			jmethodID toggleBTEnable = env->GetStaticMethodID(mediastreamerAndroidContextClass, methodName.c_str(), "()V");
 			if (toggleBTEnable != NULL) {
 					env->CallStaticVoidMethod(mediastreamerAndroidContextClass, toggleBTEnable);
-					ms_message("[Android Audio Utils] setting enable for bluetooth devices to %s", (enable) ? "true" : "false");
+					ms_debug("[Android Audio Utils] setting enable for bluetooth devices to %s", (enable) ? "true" : "false");
 			}
 			env->DeleteLocalRef(mediastreamerAndroidContextClass);
 		}
@@ -196,6 +240,36 @@ void ms_android_hack_volume(JNIEnv *env) {
 	}
 }
 
+static bool ms_android_is_device_type_headset(JNIEnv *env, const char *audioDeviceInfoClassName, int typeID) {
+	int androidSDK = ms_android_get_sdk_version(env);
+	if (androidSDK >= 31) {
+		if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_BLE_HEADSET")) {
+			return true;
+		}
+	}
+	if (androidSDK >= 26) {
+		if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_USB_HEADSET")) {
+			return true;
+		}
+	}
+	return typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_WIRED_HEADSET");
+}
+
+static bool ms_android_is_device_type_speaker(JNIEnv *env, const char *audioDeviceInfoClassName, int typeID) {
+	int androidSDK = ms_android_get_sdk_version(env);
+	if (androidSDK >= 31) {
+		if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_BLE_SPEAKER")) {
+			return true;
+		}
+	} 
+	if (androidSDK >= 30) {
+		if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_BUILTIN_SPEAKER_SAFE")) {
+			return true;
+		}
+	}
+	return typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_BUILTIN_SPEAKER");
+}
+
 MSSndCardDeviceType ms_android_get_device_type(JNIEnv *env, jobject deviceInfo) {
 	int typeID = -1;
 	const char * audioDeviceInfoClassName = "android/media/AudioDeviceInfo";
@@ -216,13 +290,10 @@ MSSndCardDeviceType ms_android_get_device_type(JNIEnv *env, jobject deviceInfo)
 		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_BLUETOOTH_A2DP;
 	} else if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_BUILTIN_EARPIECE")) {
 		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE;
-	} else if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_BUILTIN_SPEAKER")) {
+	} else if (ms_android_is_device_type_speaker(env, audioDeviceInfoClassName, typeID)) {
 		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_SPEAKER;
 	} else if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_BUILTIN_MIC")) {
 		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_MICROPHONE;
-	} else if ((ms_android_get_sdk_version(env) >= 26 && (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_USB_HEADSET")))
-			|| (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_WIRED_HEADSET"))) {
-		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADSET;
 	} else if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_WIRED_HEADPHONES")) {
 		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADPHONES;
 	} else if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_USB_DEVICE")) {
@@ -231,8 +302,11 @@ MSSndCardDeviceType ms_android_get_device_type(JNIEnv *env, jobject deviceInfo)
 		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_AUX_LINE;
 	} else if (typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_TELEPHONY")) {
 		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_TELEPHONY;
+	} else if (ms_android_is_device_type_headset(env, audioDeviceInfoClassName, typeID)) {
+		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADSET;
+	}  else if (ms_android_get_sdk_version(env) >= 28 && typeID == ms_android_getJVIntField(env, audioDeviceInfoClassName, "TYPE_HEARING_AID")) {
+		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEARING_AID;
 	} else {
-		deviceType = MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_UNKNOWN;
 		ms_error("[Android Audio Utils] Unknown device type for type ID %0d", typeID);
 	}
 
