@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -92,10 +93,12 @@ static AndroidReaderContext *getContext(MSFilter *f);
 /************************ MS2 filter methods           ************************/
 static int video_capture_set_fps(MSFilter *f, void *arg){
 	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
+	ms_mutex_lock(&d->mutex);
 	d->fps=*((float*)arg);
 	snprintf(d->fps_context, sizeof(d->fps_context), "Captured mean fps=%%f, expected=%f", d->fps);
 	ms_video_init_framerate_controller(&d->fpsControl, d->fps);
-	ms_video_init_average_fps(&d->averageFps, d->fps_context);
+	ms_average_fps_init(&d->averageFps, d->fps_context);
+	ms_mutex_unlock(&d->mutex);
 	return 0;
 }
 
@@ -294,7 +297,7 @@ void video_capture_preprocess(MSFilter *f){
 	ms_mutex_lock(&d->mutex);
 
 	ms_video_init_framerate_controller(&d->fpsControl, d->fps);
-	ms_video_init_average_fps(&d->averageFps, d->fps_context);
+	ms_average_fps_init(&d->averageFps, d->fps_context);
 
 	JNIEnv *env = ms_get_jni_env();
 
@@ -334,7 +337,7 @@ static void video_capture_process(MSFilter *f){
 		return;
 	}
 
-	ms_video_update_average_fps(&d->averageFps, f->ticker->time);
+	ms_average_fps_update(&d->averageFps, f->ticker->time);
 	mblk_set_timestamp_info(d->frame, f->ticker->time * 90);
 
 	ms_queue_put(f->outputs[0],d->frame);
