@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -428,3 +429,40 @@ SoundDeviceDescription* ms_devices_info_get_sound_device_description(MSDevicesIn
 	return d;
 }
 
+bool_t ms_sound_devices_description_equals(const SoundDeviceDescription *sdd1, const SoundDeviceDescription *sdd2) {
+	if (sdd1 == NULL && sdd2 == NULL) return TRUE;
+	if (sdd1 == NULL || sdd2 == NULL) return FALSE;
+
+	if (sdd1->flags != sdd2->flags) return FALSE;
+	if (sdd1->delay != sdd2->delay) return FALSE;
+	if (sdd1->recommended_rate != sdd2->recommended_rate) return FALSE;
+
+	int manufacturer_diff = 1;
+	if (sdd1->manufacturer == NULL && sdd2->manufacturer == NULL) {
+		manufacturer_diff = 0;
+	} else if (sdd1->manufacturer != NULL && sdd2->manufacturer != NULL) {
+		manufacturer_diff = strcmp(sdd1->manufacturer, sdd2->manufacturer);
+	} else {
+		return FALSE;
+	}
+
+	int model_diff = 1;
+	if (sdd1->model == NULL && sdd2->model == NULL) {
+		model_diff = 0;
+	} else if (sdd1->model != NULL && sdd2->model != NULL) {
+		model_diff = strcmp(sdd1->model, sdd2->model);
+	} else {
+		return FALSE;
+	}
+
+	int platform_diff = 1;
+	if (sdd1->platform == NULL && sdd2->platform == NULL) {
+		platform_diff = 0;
+	} else if (sdd1->platform != NULL && sdd2->platform != NULL) {
+		platform_diff = strcmp(sdd1->platform, sdd2->platform);
+	} else {
+		return FALSE;
+	}
+
+	return manufacturer_diff == 0 && model_diff == 0 && platform_diff == 0;
+}
