@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -285,7 +286,9 @@ void ms_snd_card_manager_unregister_desc(MSSndCardManager *m, MSSndCardDesc *des
 
 bool_t ms_snd_card_equals(const MSSndCard *c1, const MSSndCard *c2){
 	if (strcmp(ms_snd_card_get_string_id(c1), ms_snd_card_get_string_id(c2)) == 0
-		&& c1->capabilities == c2->capabilities){
+		&& c1->capabilities == c2->capabilities
+		&& ms_sound_devices_description_equals(c1->device_description, c2->device_description)
+	) {
 		return TRUE;
 	}
 	return FALSE;
@@ -361,6 +364,7 @@ MSSndCard * ms_snd_card_new_with_name(MSSndCardDesc *desc,const char* name) {
 	obj->device_type=MS_SND_CARD_DEVICE_TYPE_UNKNOWN;
 	obj->capabilities=MS_SND_CARD_CAP_CAPTURE|MS_SND_CARD_CAP_PLAYBACK;
 	obj->streamType=MS_SND_CARD_STREAM_VOICE;
+	obj->device_description=NULL;
 	if (desc->init!=NULL)
 		desc->init(obj);
 	return obj;
@@ -392,6 +396,8 @@ const char * ms_snd_card_device_type_to_string(const MSSndCardDeviceType type){
 			return "Bluetooth";
 		case MS_SND_CARD_DEVICE_TYPE_BLUETOOTH_A2DP:
 			return "Bluetooth A2DP";
+		case MS_SND_CARD_DEVICE_TYPE_HEARING_AID:
+			return "Hearing Aid";
 		case MS_SND_CARD_DEVICE_TYPE_UNKNOWN:
 			return "Unknown";
 		default:
@@ -573,6 +579,7 @@ void ms_snd_card_unref(MSSndCard *sndCard) {
 		if (sndCard->desc->uninit) sndCard->desc->uninit(sndCard);
 		if (sndCard->name != NULL) ms_free(sndCard->name);
 		if (sndCard->id != NULL) ms_free(sndCard->id);
+		sndCard->device_description = NULL;
 		ms_free(sndCard);
 	}
 }
