@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -324,6 +325,12 @@ static int ms_set_srtp_crypto_policy(MSCryptoSuite suite, crypto_policy_t *polic
 		case MS_AES_256_SHA1_32:
 			crypto_policy_set_aes_cm_256_hmac_sha1_32(policy);
 			break;
+		case MS_AEAD_AES_128_GCM:
+			srtp_crypto_policy_set_aes_gcm_128_16_auth(policy);
+			break;
+		case MS_AEAD_AES_256_GCM:
+			srtp_crypto_policy_set_aes_gcm_256_16_auth(policy);
+			break;
 		case MS_CRYPTO_SUITE_INVALID:
 			return -1;
 			break;
@@ -331,17 +338,53 @@ static int ms_set_srtp_crypto_policy(MSCryptoSuite suite, crypto_policy_t *polic
 	return 0;
 }
 
+const char * ms_crypto_suite_to_string(MSCryptoSuite suite) {
+	switch(suite) {
+		case MS_CRYPTO_SUITE_INVALID:
+			return "<invalid-or-unsupported-suite>";
+			break;
+		case MS_AES_128_SHA1_80:
+			return "AES_CM_128_HMAC_SHA1_80";
+			break;
+		case MS_AES_128_SHA1_32:
+			return "AES_CM_128_HMAC_SHA1_32";
+			break;
+		case MS_AES_128_SHA1_80_NO_AUTH:
+			return "AES_CM_128_HMAC_SHA1_80 UNAUTHENTICATED_SRTP";
+			break;
+		case MS_AES_128_SHA1_32_NO_AUTH:
+			return "AES_CM_128_HMAC_SHA1_32 UNAUTHENTICATED_SRTP";
+			break;
+		case MS_AES_128_SHA1_80_SRTP_NO_CIPHER:
+			return "AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTP";
+			break;
+		case MS_AES_128_SHA1_80_SRTCP_NO_CIPHER:
+			return "AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTCP";
+			break;
+		case MS_AES_128_SHA1_80_NO_CIPHER:
+			return "AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTP UNENCRYPTED_SRTCP";
+			break;
+		case MS_AES_256_SHA1_80:
+			return "AES_256_CM_HMAC_SHA1_80";
+			break;
+		case MS_AES_CM_256_SHA1_80:
+			return "AES_CM_256_HMAC_SHA1_80";
+			break;
+		case MS_AES_256_SHA1_32:
+			return "AES_256_CM_HMAC_SHA1_32";
+			break;
+		case MS_AEAD_AES_128_GCM:
+			return "AEAD_AES_128_GCM";
+			break;
+		case MS_AEAD_AES_256_GCM:
+			return "AEAD_AES_256_GCM";
+			break;
+	}
+	return "<invalid-or-unsupported-suite>";
+}
+
 static int ms_srtp_add_or_update_stream(srtp_t session, const srtp_policy_t *policy)
 {
-/*
-	err_status_t status;
-	// Update streams if the session was already initialized
-	if (session->stream_template) {
-		status = srtp_update_stream(session, policy);
-	} else {
-		status = srtp_add_stream(session, policy);
-	}
-*/
 
 	err_status_t status = srtp_update_stream(session, policy);
 	if (status != 0) {
