@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -47,67 +48,41 @@ struct _MSZrtpContext{
 /***** LOCAL FUNCTIONS                     *****/
 /***********************************************/
 
-/********************/
-/* Helper functions */
-
-/* trace functions: bzrtp algo code to string */
-static const char *bzrtp_hash_toString(uint8_t hashAlgo) {
-	switch(hashAlgo) {
-		case(ZRTP_UNSET_ALGO): return "unset";
-		case(ZRTP_HASH_S256): return "SHA-256";
-		case(ZRTP_HASH_S384): return "SHA-384";
-		case(ZRTP_HASH_N256): return "SHA3-256";
-		case(ZRTP_HASH_N384): return "SHA3-384";
-		default: return "Unknown Algo";
-	}
-}
-
-static const char *bzrtp_keyAgreement_toString(uint8_t keyAgreementAlgo) {
-	switch(keyAgreementAlgo) {
-		case(ZRTP_UNSET_ALGO): return "unset";
-		case(ZRTP_KEYAGREEMENT_DH2k): return "DHM-2048";
-		case(ZRTP_KEYAGREEMENT_EC25): return "ECDH-256";
-		case(ZRTP_KEYAGREEMENT_DH3k): return "DHM-3072";
-		case(ZRTP_KEYAGREEMENT_EC38): return "ECDH-384";
-		case(ZRTP_KEYAGREEMENT_EC52): return "ECDH-521";
-		case(ZRTP_KEYAGREEMENT_X255): return "ECDH-255";
-		case(ZRTP_KEYAGREEMENT_X448): return "ECDH-448";
-		case(ZRTP_KEYAGREEMENT_Prsh): return "PreShared";
-		case(ZRTP_KEYAGREEMENT_Mult): return "MultiStream";
-		default: return "Unknown Algo";
-	}
-}
-
-static const char *bzrtp_cipher_toString(uint8_t cipherAlgo) {
-	switch(cipherAlgo) {
-		case(ZRTP_UNSET_ALGO): return "unset";
-		case(ZRTP_CIPHER_AES1): return "AES-128";
-		case(ZRTP_CIPHER_AES2): return "AES-192";
-		case(ZRTP_CIPHER_AES3): return "AES-256";
-		case(ZRTP_CIPHER_2FS1): return "TwoFish-128";
-		case(ZRTP_CIPHER_2FS2): return "TwoFish-192";
-		case(ZRTP_CIPHER_2FS3): return "TwoFish-256";
-		default: return "Unknown Algo";
-	}
-}
-
-static const char *bzrtp_authtag_toString(uint8_t authtagAlgo) {
-	switch(authtagAlgo) {
-		case(ZRTP_UNSET_ALGO): return "unset";
-		case(ZRTP_AUTHTAG_HS32): return "HMAC-SHA1-32";
-		case(ZRTP_AUTHTAG_HS80): return "HMAC-SHA1-80";
-		case(ZRTP_AUTHTAG_SK32): return "Skein-32";
-		case(ZRTP_AUTHTAG_SK64): return "Skein-64";
-		default: return "Unknown Algo";
-	}
-}
-
-static const char *bzrtp_sas_toString(uint8_t sasAlgo) {
-	switch(sasAlgo) {
-		case(ZRTP_UNSET_ALGO): return "unset";
-		case(ZRTP_SAS_B32): return "Base32";
-		case(ZRTP_SAS_B256): return "PGP-WordList";
-		default: return "Unknown Algo";
+#define PACKET_INFO_MAX_SIZE 256
+/**
+ * @brief extract the informations from the packet header and print them in the given info string
+ *     - packet Type (if available)
+ *     - is packet fragmented? If yes:
+ *        - messageId
+ *        - fragment size and offset
+ *
+ * @param[in]	packet	The ZRTP packet to analyse
+ * @param[out]	info	The string to print the informations
+ */
+static void ms_zrtp_getPacketInfo(const uint8_t *packet, char info[PACKET_INFO_MAX_SIZE]) {
+	uint16_t seqNum = ((uint16_t)packet[2]<<8) + ((uint16_t)packet[3]);
+	if (packet[0] == 0x10) { /* packet is not fragmented */
+		snprintf(info, PACKET_INFO_MAX_SIZE, "message %.8s with seqNum %x", packet+16, seqNum);
+	} else if (packet[0] == 0x11) { /* message fragment */
+		uint16_t offset = ((uint16_t)packet[16]<<8)+((uint16_t)packet[17]);
+		if (offset == 0) { /* we can retrieve the packet type */
+			snprintf(info, PACKET_INFO_MAX_SIZE, "fragmented message %.8s id %04x offset %d fragSize %d seqNum %x",
+					packet+24,
+					((uint16_t)packet[12]<<8)+((uint16_t)packet[13]), /* message Id */
+					offset,
+					((uint16_t)packet[18]<<8)+((uint16_t)packet[19]), /* frag size */
+					seqNum
+					);
+		} else {
+			snprintf(info, PACKET_INFO_MAX_SIZE, "fragmented message id %04x offset %d fragSize %d seqNum %x",
+					((uint16_t)packet[12]<<8)+((uint16_t)packet[13]), /* message Id */
+					offset,
+					((uint16_t)packet[18]<<8)+((uint16_t)packet[19]), /* frag size */
+					seqNum
+					);
+		}
+	} else {
+		snprintf(info, PACKET_INFO_MAX_SIZE, "invalid packet");
 	}
 }
 /*****************************************/
@@ -146,6 +121,16 @@ static int ms_zrtp_statusMessage(void *clientData, const uint8_t messageLevel, c
 			/* Do not forward this message upward, just log it, shall we use this to prevent LIME keys creation as it is unlikely peer implement LIME? */
 			ms_warning("Peer ZRTP engine version is not BZRTP and would not allow LIME to work correctly, peer ZRTP engine identifies itself as %.16s", messageString==NULL?"NULL":messageString);
 			break;
+		case BZRTP_MESSAGE_PEERREQUESTGOCLEAR:
+			ev=ortp_event_new(ORTP_EVENT_ZRTP_PEER_REQUEST_GOCLEAR);
+			rtp_session_dispatch_event(userData->stream_sessions->rtp_session, ev);
+			ms_message("Zrtp Event dispatched : Send ZRTP GoClear");
+			break;
+		case BZRTP_MESSAGE_PEERACKGOCLEAR:
+			ev=ortp_event_new(ORTP_EVENT_ZRTP_PEER_ACK_GOCLEAR);
+			rtp_session_dispatch_event(userData->stream_sessions->rtp_session, ev);
+			ms_message("Zrtp Event dispatched : Acknowledge ZRTP GoClear");
+			break;
 		default:
 			/* unexepected message, do nothing, just log it as a warning */
 			ms_warning("Zrtp Message Unknown : Level %d Id %d message %s", messageLevel, messageId, messageString==NULL?"NULL":messageString);
@@ -171,7 +156,9 @@ static int32_t ms_zrtp_sendDataZRTP (void *clientData, const uint8_t* data, uint
 	RtpTransport *rtpt=NULL;
 	mblk_t *msg;
 
-	ms_message("ZRTP Send packet type %.8s on rtp session [%p]", data+16, session);
+	char packetInfo[PACKET_INFO_MAX_SIZE];
+	ms_zrtp_getPacketInfo(data, packetInfo);
+	ms_message("ZRTP Send %s of size %d on rtp session [%p]", packetInfo, length, session);
 
 	/* get RTP transport from session */
 	rtp_session_get_transports(session,&rtpt,NULL);
@@ -211,7 +198,7 @@ static int32_t ms_zrtp_srtpSecretsAvailable(void* clientData, const bzrtpSrtpSec
 		ms_fatal("unsupported cipher algorithm by srtp");
 	}
 
-	ms_message("ZRTP secrets are ready for %s; auth tag algo is %s and cipher algo is %s", (part==ZRTP_SRTP_SECRETS_FOR_SENDER)?"sender":"receiver", bzrtp_authtag_toString(secrets->authTagAlgo), bzrtp_cipher_toString(secrets->cipherAlgo));
+	ms_message("ZRTP secrets are ready for %s; auth tag algo is %s and cipher algo is %s", (part==ZRTP_SRTP_SECRETS_FOR_SENDER)?"sender":"receiver", bzrtp_algoToString(secrets->authTagAlgo), bzrtp_algoToString(secrets->cipherAlgo));
 
 
 	if (part==ZRTP_SRTP_SECRETS_FOR_RECEIVER) {
@@ -263,6 +250,64 @@ static int32_t ms_zrtp_srtpSecretsAvailable(void* clientData, const bzrtpSrtpSec
 	return 0;
 }
 
+/**
+ * @brief: convert bzrtp uint8_t mapped algo Id to ms enum mapped algo Id
+ *
+ * @param[in]	algo	the uint8_t bzrtp mapped(in bzrtp.h) algo id
+ *
+ * @return the ms enum mapped (in zrtp.h) algo id
+ */
+
+static int ms_zrtp_getAlgoId(uint8_t algo) {
+	switch(algo) {
+
+		case(ZRTP_HASH_S256): return MS_ZRTP_HASH_S256;
+		case(ZRTP_HASH_S384): return MS_ZRTP_HASH_S384;
+		case(ZRTP_HASH_S512): return MS_ZRTP_HASH_S512;
+		case(ZRTP_HASH_N256): return MS_ZRTP_HASH_N256;
+		case(ZRTP_HASH_N384): return MS_ZRTP_HASH_N384;
+
+		case(ZRTP_CIPHER_AES1): return MS_ZRTP_CIPHER_AES1;
+		case(ZRTP_CIPHER_AES2): return MS_ZRTP_CIPHER_AES2;
+		case(ZRTP_CIPHER_AES3): return MS_ZRTP_CIPHER_AES3;
+		case(ZRTP_CIPHER_2FS1): return MS_ZRTP_CIPHER_2FS1;
+		case(ZRTP_CIPHER_2FS2): return MS_ZRTP_CIPHER_2FS2;
+		case(ZRTP_CIPHER_2FS3): return MS_ZRTP_CIPHER_2FS3;
+
+		case(ZRTP_AUTHTAG_HS32): return MS_ZRTP_AUTHTAG_HS32;
+		case(ZRTP_AUTHTAG_HS80): return MS_ZRTP_AUTHTAG_HS80;
+		case(ZRTP_AUTHTAG_SK32): return MS_ZRTP_AUTHTAG_SK32;
+		case(ZRTP_AUTHTAG_SK64): return MS_ZRTP_AUTHTAG_SK64;
+
+		case(ZRTP_KEYAGREEMENT_DH2k): return MS_ZRTP_KEY_AGREEMENT_DH2K;
+		case(ZRTP_KEYAGREEMENT_EC25): return MS_ZRTP_KEY_AGREEMENT_EC25;
+		case(ZRTP_KEYAGREEMENT_DH3k): return MS_ZRTP_KEY_AGREEMENT_DH3K;
+		case(ZRTP_KEYAGREEMENT_EC38): return MS_ZRTP_KEY_AGREEMENT_EC38;
+		case(ZRTP_KEYAGREEMENT_EC52): return MS_ZRTP_KEY_AGREEMENT_EC52;
+		case(ZRTP_KEYAGREEMENT_X255): return MS_ZRTP_KEY_AGREEMENT_X255;
+		case(ZRTP_KEYAGREEMENT_X448): return MS_ZRTP_KEY_AGREEMENT_X448;
+		case(ZRTP_KEYAGREEMENT_K255): return MS_ZRTP_KEY_AGREEMENT_K255;
+		case(ZRTP_KEYAGREEMENT_K448): return MS_ZRTP_KEY_AGREEMENT_K448;
+		case(ZRTP_KEYAGREEMENT_KYB1): return MS_ZRTP_KEY_AGREEMENT_KYB1;
+		case(ZRTP_KEYAGREEMENT_KYB2): return MS_ZRTP_KEY_AGREEMENT_KYB2;
+		case(ZRTP_KEYAGREEMENT_KYB3): return MS_ZRTP_KEY_AGREEMENT_KYB3;
+		case(ZRTP_KEYAGREEMENT_HQC1): return MS_ZRTP_KEY_AGREEMENT_HQC1;
+		case(ZRTP_KEYAGREEMENT_HQC2): return MS_ZRTP_KEY_AGREEMENT_HQC2;
+		case(ZRTP_KEYAGREEMENT_HQC3): return MS_ZRTP_KEY_AGREEMENT_HQC3;
+		case(ZRTP_KEYAGREEMENT_K255_KYB512): return MS_ZRTP_KEY_AGREEMENT_K255_KYB512;
+		case(ZRTP_KEYAGREEMENT_K255_HQC128): return MS_ZRTP_KEY_AGREEMENT_K255_HQC128;
+		case(ZRTP_KEYAGREEMENT_K448_KYB1024): return MS_ZRTP_KEY_AGREEMENT_K448_KYB1024;
+		case(ZRTP_KEYAGREEMENT_K448_HQC256): return MS_ZRTP_KEY_AGREEMENT_K448_HQC256;
+		case(ZRTP_KEYAGREEMENT_K255_KYB512_HQC128): return MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128;
+		case(ZRTP_KEYAGREEMENT_K448_KYB1024_HQC256): return MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256;
+
+		case(ZRTP_SAS_B32): return MS_ZRTP_SAS_B32;
+		case(ZRTP_SAS_B256): return MS_ZRTP_SAS_B256;
+
+		default: return 0;
+	}
+}
+
 /**
  * @brief Switch on the security.
  *
@@ -292,11 +337,16 @@ static int ms_zrtp_startSrtpSession(void *clientData,  const bzrtpSrtpSecrets_t
 		eventData=ortp_event_get_data(ev);
 		// support both b32 and b256 format SAS strings
 		snprintf(eventData->info.zrtp_info.sas, sizeof(eventData->info.zrtp_info.sas), "%s", secrets->sas);
-		eventData->info.zrtp_info.verified=(verified != 0) ? TRUE : FALSE;
-		eventData->info.zrtp_info.cache_mismatch=( secrets->cacheMismatch != 0) ? TRUE : FALSE;
+		eventData->info.zrtp_info.verified = (verified != 0) ? TRUE : FALSE;
+		eventData->info.zrtp_info.cache_mismatch = ( secrets->cacheMismatch != 0) ? TRUE : FALSE;
+		eventData->info.zrtp_info.cipherAlgo = ms_zrtp_getAlgoId(secrets->cipherAlgo);
+		eventData->info.zrtp_info.keyAgreementAlgo = ms_zrtp_getAlgoId(secrets->keyAgreementAlgo);
+		eventData->info.zrtp_info.hashAlgo = ms_zrtp_getAlgoId(secrets->hashAlgo);
+		eventData->info.zrtp_info.authTagAlgo = ms_zrtp_getAlgoId(secrets->authTagAlgo);
+		eventData->info.zrtp_info.sasAlgo = ms_zrtp_getAlgoId(secrets->sasAlgo);
 		rtp_session_dispatch_event(userData->stream_sessions->rtp_session, ev);
 		ms_message("ZRTP secrets on: SAS is %.32s previously verified %s on session [%p]", secrets->sas, verified == 0 ? "no" : "yes", userData->stream_sessions);
-		ms_message("ZRTP algo used during negotiation: Cipher: %s - KeyAgreement: %s - Hash: %s - AuthTag: %s - Sas Rendering: %s", bzrtp_cipher_toString(secrets->cipherAlgo), bzrtp_keyAgreement_toString(secrets->keyAgreementAlgo), bzrtp_hash_toString(secrets->hashAlgo), bzrtp_authtag_toString(secrets->authTagAlgo), bzrtp_sas_toString(secrets->sasAlgo));
+		ms_message("ZRTP algo used during negotiation: Cipher: %s - KeyAgreement: %s - Hash: %s - AuthTag: %s - Sas Rendering: %s", bzrtp_algoToString(secrets->cipherAlgo), bzrtp_algoToString(secrets->keyAgreementAlgo), bzrtp_algoToString(secrets->hashAlgo), bzrtp_algoToString(secrets->authTagAlgo), bzrtp_algoToString(secrets->sasAlgo));
 	}
 
 	ev=ortp_event_new(ORTP_EVENT_ZRTP_ENCRYPTION_CHANGED);
@@ -421,15 +471,21 @@ static int ms_zrtp_rtp_process_on_receive(RtpTransportModifier *t, mblk_t *msg){
 	}
 
 	// display received message
-	ms_message("ZRTP Receive packet type %.8s on rtp session [%p]", rtp+16, t->session);
+	char packetInfo[PACKET_INFO_MAX_SIZE];
+	ms_zrtp_getPacketInfo(rtp, packetInfo);
+	ms_message("ZRTP Receive %s of size %d on rtp session [%p]", packetInfo, msgLength, t->session);
 
 	// check if the ZRTP channel is started, if not(ZRTP not set on our side but incoming zrtp packets), start it
 	if (userData->autoStart && bzrtp_getChannelStatus(zrtpContext, userData->self_ssrc) == BZRTP_CHANNEL_INITIALISED) {
+		ms_message("ZRTP autostart channel on rtp session [%p]", t->session);
 		bzrtp_startChannelEngine(zrtpContext, userData->self_ssrc);
 	}
 
 	// send ZRTP packet to engine
-	bzrtp_processMessage(zrtpContext, userData->self_ssrc, rtp, msgLength);
+	int ret = bzrtp_processMessage(zrtpContext, userData->self_ssrc, rtp, msgLength);
+	if (ret != 0) {
+		ms_message("ZRTP packet %s processing returns %04x on rtp session [%p]", packetInfo, ret, t->session);
+	}
 	return 0;
 }
 
@@ -490,6 +546,7 @@ static void set_hash_suites(bzrtpContext_t *ctx, const MSZrtpHash *hashes, const
 			case MS_ZRTP_HASH_INVALID: break;
 			case MS_ZRTP_HASH_S256: bzrtpHashes[bzrtpCount++] = ZRTP_HASH_S256; break;
 			case MS_ZRTP_HASH_S384: bzrtpHashes[bzrtpCount++] = ZRTP_HASH_S384; break;
+			case MS_ZRTP_HASH_S512: bzrtpHashes[bzrtpCount++] = ZRTP_HASH_S512; break;
 			case MS_ZRTP_HASH_N256: bzrtpHashes[bzrtpCount++] = ZRTP_HASH_N256; break;
 			case MS_ZRTP_HASH_N384: bzrtpHashes[bzrtpCount++] = ZRTP_HASH_N384; break;
 		}
@@ -543,14 +600,28 @@ static void set_key_agreement_suites(bzrtpContext_t *ctx, const MSZrtpKeyAgreeme
 
 	for (i=0; i < count; i++) {
 		switch (keyAgreements[i]) {
-			case MS_ZRTP_KEY_AGREEMENT_INVALID: break;
-			case MS_ZRTP_KEY_AGREEMENT_DH2K:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_DH2k; break;
-			case MS_ZRTP_KEY_AGREEMENT_DH3K:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_DH3k; break;
-			case MS_ZRTP_KEY_AGREEMENT_EC25:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC25; break;
-			case MS_ZRTP_KEY_AGREEMENT_EC38:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC38; break;
-			case MS_ZRTP_KEY_AGREEMENT_EC52:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC52; break;
-			case MS_ZRTP_KEY_AGREEMENT_X255:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_X255; break;
-			case MS_ZRTP_KEY_AGREEMENT_X448:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_X448; break;
+			case MS_ZRTP_KEY_AGREEMENT_INVALID:			break;
+			case MS_ZRTP_KEY_AGREEMENT_DH2K:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_DH2k; break;
+			case MS_ZRTP_KEY_AGREEMENT_DH3K:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_DH3k; break;
+			case MS_ZRTP_KEY_AGREEMENT_EC25:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC25; break;
+			case MS_ZRTP_KEY_AGREEMENT_EC38:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC38; break;
+			case MS_ZRTP_KEY_AGREEMENT_EC52:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC52; break;
+			case MS_ZRTP_KEY_AGREEMENT_X255:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_X255; break;
+			case MS_ZRTP_KEY_AGREEMENT_X448:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_X448; break;
+			case MS_ZRTP_KEY_AGREEMENT_K255:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K255; break;
+			case MS_ZRTP_KEY_AGREEMENT_K448:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K448; break;
+			case MS_ZRTP_KEY_AGREEMENT_KYB1:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_KYB1; break;
+			case MS_ZRTP_KEY_AGREEMENT_KYB2:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_KYB2; break;
+			case MS_ZRTP_KEY_AGREEMENT_KYB3:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_KYB3; break;
+			case MS_ZRTP_KEY_AGREEMENT_HQC1:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_HQC1; break;
+			case MS_ZRTP_KEY_AGREEMENT_HQC2:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_HQC2; break;
+			case MS_ZRTP_KEY_AGREEMENT_HQC3:			bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_HQC3; break;
+			case MS_ZRTP_KEY_AGREEMENT_K255_KYB512:     bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K255_KYB512; break;
+			case MS_ZRTP_KEY_AGREEMENT_K255_HQC128:     bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K255_HQC128; break;
+			case MS_ZRTP_KEY_AGREEMENT_K448_KYB1024:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K448_KYB1024; break;
+			case MS_ZRTP_KEY_AGREEMENT_K448_HQC256:     bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K448_HQC256; break;
+			case MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128:     bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K255_KYB512_HQC128; break;
+			case MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_K448_KYB1024_HQC256; break;
 		}
 	}
 
@@ -588,10 +659,6 @@ void ms_zrtp_set_stream_sessions(MSZrtpContext *zrtp_context, MSMediaStreamSessi
 /* header declared in include/mediastreamer2/zrtp.h */
 bool_t ms_zrtp_available(){return TRUE;}
 
-void ms_zrtp_context_reset(MSZrtpContext* context) {
-	bzrtp_resetBzrtpContext(context->zrtpContext);
-}
-
 MSZrtpContext* ms_zrtp_context_new(MSMediaStreamSessions *sessions, MSZrtpParams *params) {
 	MSZrtpContext *userData;
 	bzrtpContext_t *context;
@@ -599,6 +666,11 @@ MSZrtpContext* ms_zrtp_context_new(MSMediaStreamSessions *sessions, MSZrtpParams
 
 	ms_message("Creating ZRTP engine on rtp session [%p] ssrc 0x%x",sessions->rtp_session, sessions->rtp_session->snd.ssrc);
 	context = bzrtp_createBzrtpContext();
+#ifdef HAVE_GOCLEAR
+	bzrtp_setFlags(context, BZRTP_SELF_ACCEPT_GOCLEAR, params->acceptGoClear);
+#else
+	bzrtp_setFlags(context, BZRTP_SELF_ACCEPT_GOCLEAR, FALSE);
+#endif // HAVE_GOCLEAR
 
 	if (params->zidCacheDB != NULL && params->selfUri != NULL && params->peerUri) { /* to enable cache we need a self and peer uri and a pointer to the sqlite cache DB */
 		/*enabling cache*/
@@ -645,7 +717,8 @@ MSZrtpContext* ms_zrtp_multistream_new(MSMediaStreamSessions *sessions, MSZrtpCo
 	int retval;
 	MSZrtpContext *userData;
 	if ((retval = bzrtp_addChannel(activeContext->zrtpContext, sessions->rtp_session->snd.ssrc)) != 0) {
-		ms_warning("ZRTP could't add stream, returns %x", retval);
+		ms_error("ZRTP could't add stream, returns %x", retval);
+		return NULL;
 	}
 
 	ms_message("Initializing multistream ZRTP context on rtp session [%p] ssrc 0x%x",sessions->rtp_session, sessions->rtp_session->snd.ssrc);
@@ -659,6 +732,10 @@ MSZrtpContext* ms_zrtp_multistream_new(MSMediaStreamSessions *sessions, MSZrtpCo
 	return ms_zrtp_configure_context(userData, sessions->rtp_session);
 }
 
+void ms_zrtp_enable_go_clear(MSZrtpContext *ctx, bool_t enable) {
+	bzrtp_setFlags(ctx->zrtpContext, BZRTP_SELF_ACCEPT_GOCLEAR, enable);
+}
+
 int ms_zrtp_channel_start(MSZrtpContext *ctx) {
 	int retval;
 	ms_message("Starting ZRTP engine on rtp session [%p] ssrc 0x%x",ctx->stream_sessions->rtp_session, ctx->self_ssrc);
@@ -677,7 +754,9 @@ int ms_zrtp_channel_start(MSZrtpContext *ctx) {
 
 void ms_zrtp_context_destroy(MSZrtpContext *ctx) {
 	ms_message("Stopping ZRTP context on session [%p]", ctx->stream_sessions ? ctx->stream_sessions->rtp_session : NULL);
-	bzrtp_destroyBzrtpContext(ctx->zrtpContext, ctx->self_ssrc);
+	if (ctx->zrtpContext) {
+		bzrtp_destroyBzrtpContext(ctx->zrtpContext, ctx->self_ssrc);
+	}
 	ms_free(ctx);
 	ms_message("ZRTP context destroyed");
 }
@@ -773,12 +852,61 @@ int ms_zrtp_cache_migration(void *cacheXmlPtr, void *cacheSqlite, const char *se
 	}
 }
 
-#else /* HAVE_ZRTP */
+uint8_t ms_zrtp_available_key_agreement(MSZrtpKeyAgreement algos[256]) {
+	uint8_t bzrtpAlgos[256];
+	uint8_t nbAlgos = bzrtp_available_key_agreement(bzrtpAlgos);
+	uint8_t i=0;
+	uint8_t count=0;
+	for (i=0; i < nbAlgos; i++) {
+		switch (bzrtpAlgos[i]) {
+			case ZRTP_KEYAGREEMENT_DH2k: algos[count++] = MS_ZRTP_KEY_AGREEMENT_DH2K; break;
+			case ZRTP_KEYAGREEMENT_X255: algos[count++] = MS_ZRTP_KEY_AGREEMENT_X255; break;
+			case ZRTP_KEYAGREEMENT_K255: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K255; break;
+			case ZRTP_KEYAGREEMENT_EC25: algos[count++] = MS_ZRTP_KEY_AGREEMENT_EC25; break;
+			case ZRTP_KEYAGREEMENT_X448: algos[count++] = MS_ZRTP_KEY_AGREEMENT_X448; break;
+			case ZRTP_KEYAGREEMENT_K448: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K448; break;
+			case ZRTP_KEYAGREEMENT_DH3k: algos[count++] = MS_ZRTP_KEY_AGREEMENT_DH3K; break;
+			case ZRTP_KEYAGREEMENT_EC38: algos[count++] = MS_ZRTP_KEY_AGREEMENT_EC38; break;
+			case ZRTP_KEYAGREEMENT_EC52: algos[count++] = MS_ZRTP_KEY_AGREEMENT_EC52; break;
+			case ZRTP_KEYAGREEMENT_KYB1: algos[count++] = MS_ZRTP_KEY_AGREEMENT_KYB1; break;
+			case ZRTP_KEYAGREEMENT_KYB2: algos[count++] = MS_ZRTP_KEY_AGREEMENT_KYB2; break;
+			case ZRTP_KEYAGREEMENT_KYB3: algos[count++] = MS_ZRTP_KEY_AGREEMENT_KYB3; break;
+			case ZRTP_KEYAGREEMENT_HQC1: algos[count++] = MS_ZRTP_KEY_AGREEMENT_HQC1; break;
+			case ZRTP_KEYAGREEMENT_HQC2: algos[count++] = MS_ZRTP_KEY_AGREEMENT_HQC2; break;
+			case ZRTP_KEYAGREEMENT_HQC3: algos[count++] = MS_ZRTP_KEY_AGREEMENT_HQC3; break;
+			case ZRTP_KEYAGREEMENT_K255_KYB512: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K255_KYB512; break;
+			case ZRTP_KEYAGREEMENT_K255_HQC128: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K255_HQC128; break;
+			case ZRTP_KEYAGREEMENT_K448_KYB1024: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K448_KYB1024; break;
+			case ZRTP_KEYAGREEMENT_K448_HQC256: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K448_HQC256; break;
+			case ZRTP_KEYAGREEMENT_K255_KYB512_HQC128: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128; break;
+			case ZRTP_KEYAGREEMENT_K448_KYB1024_HQC256: algos[count++] = MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256; break;
+			default: break;
+		}
+	}
 
-void ms_zrtp_context_reset(MSZrtpContext* context) {
-	ms_message("ZRTP is disabled");
+	return count;
 }
 
+bool_t ms_zrtp_is_PQ_available(void) {
+	return bzrtp_is_PQ_available();
+}
+
+//#ifdef HAVE_GOCLEAR
+int ms_zrtp_send_go_clear(MSZrtpContext *ctx){
+    return bzrtp_sendGoClear(ctx->zrtpContext, ctx->self_ssrc);
+}
+
+int ms_zrtp_confirm_go_clear(MSZrtpContext *ctx){
+	return bzrtp_confirmGoClear(ctx->zrtpContext, ctx->self_ssrc);
+}
+
+int ms_zrtp_back_to_secure_mode(MSZrtpContext *ctx){
+    return bzrtp_backToSecureMode(ctx->zrtpContext, ctx->self_ssrc);
+}
+//#endif /* HAVE_GOCLEAR */
+
+#else /* HAVE_ZRTP */
+
 MSZrtpContext* ms_zrtp_context_new(MSMediaStreamSessions *sessions, MSZrtpParams *params){
 	ms_message("ZRTP is disabled");
 	return NULL;
@@ -789,6 +917,7 @@ MSZrtpContext* ms_zrtp_multistream_new(MSMediaStreamSessions *sessions, MSZrtpCo
 	return NULL;
 }
 
+void ms_zrtp_enable_go_clear(MSZrtpContext *ctx, bool_t enable) {};
 int ms_zrtp_channel_start(MSZrtpContext *ctx) { return 0;}
 bool_t ms_zrtp_available(){return FALSE;}
 void ms_zrtp_sas_verified(MSZrtpContext* ctx){}
@@ -800,9 +929,16 @@ int ms_zrtp_transport_modifier_new(MSZrtpContext* ctx, RtpTransportModifier **rt
 void ms_zrtp_transport_modifier_destroy(RtpTransportModifier *tp)  {}
 void ms_zrtp_set_stream_sessions(MSZrtpContext *zrtp_context, MSMediaStreamSessions *stream_sessions) {}
 int ms_zrtp_getHelloHash(MSZrtpContext* ctx, uint8_t *output, size_t outputLength) {return 0;}
+int ms_zrtp_setAuxiliarySharedSecret(MSZrtpContext *ctx, const uint8_t *auxSharedSecret, size_t auxSharedSecretLength) {return 0;}
+uint8_t ms_zrtp_getAuxiliarySharedSecretMismatch(MSZrtpContext *ctx) {return 0;}
 int ms_zrtp_setPeerHelloHash(MSZrtpContext *ctx, uint8_t *peerHelloHashHexString, size_t peerHelloHashHexStringLength) {return 0;}
 int ms_zrtp_initCache(void *db, bctbx_mutex_t *dbMutex){return 0;}
 int ms_zrtp_cache_migration(void *cacheXmlPtr, void *cacheSqlite, const char *selfURI) {return 0;}
+uint8_t ms_zrtp_available_key_agreement(MSZrtpKeyAgreement algos[256]) { return 0;}
+bool_t ms_zrtp_is_PQ_available(void) { return FALSE;}
+int ms_zrtp_send_go_clear(MSZrtpContext *ctx){return 0;}
+int ms_zrtp_confirm_go_clear(MSZrtpContext *ctx){return 0;}
+int ms_zrtp_back_to_secure_mode(MSZrtpContext *ctx){return 0;}
 #endif /* HAVE_ZRTP */
 
 #define STRING_COMPARE_RETURN(string, value)\
@@ -820,6 +956,7 @@ int ms_zrtp_cache_migration(void *cacheXmlPtr, void *cacheSqlite, const char *se
 MS2_PUBLIC MSZrtpHash ms_zrtp_hash_from_string(const char* str) {
 	STRING_COMPARE_RETURN(str, MS_ZRTP_HASH_S256);
 	STRING_COMPARE_RETURN(str, MS_ZRTP_HASH_S384);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_HASH_S512);
 	STRING_COMPARE_RETURN(str, MS_ZRTP_HASH_N256);
 	STRING_COMPARE_RETURN(str, MS_ZRTP_HASH_N384);
 	return MS_ZRTP_HASH_INVALID;
@@ -830,6 +967,7 @@ MS2_PUBLIC const char* ms_zrtp_hash_to_string(const MSZrtpHash hash) {
 		CASE_RETURN_STRING(MS_ZRTP_HASH_INVALID);\
 		CASE_RETURN_STRING(MS_ZRTP_HASH_S256);\
 		CASE_RETURN_STRING(MS_ZRTP_HASH_S384);\
+		CASE_RETURN_STRING(MS_ZRTP_HASH_S512);\
 		CASE_RETURN_STRING(MS_ZRTP_HASH_N256);\
 		CASE_RETURN_STRING(MS_ZRTP_HASH_N384);\
 	);
@@ -884,6 +1022,20 @@ MSZrtpKeyAgreement ms_zrtp_key_agreement_from_string(const char* str) {
 	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_EC52);
 	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_X255);
 	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_X448);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K255);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K448);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_KYB1);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_KYB2);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_KYB3);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_HQC1);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_HQC2);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_HQC3);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K255_KYB512);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K255_HQC128);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K448_KYB1024);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K448_HQC256);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256);
 	return MS_ZRTP_KEY_AGREEMENT_INVALID;
 }
 
@@ -897,6 +1049,20 @@ const char* ms_zrtp_key_agreement_to_string(const MSZrtpKeyAgreement keyAgreemen
 		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_EC52);\
 		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_X255);\
 		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_X448);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K255);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K448);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_KYB1);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_KYB2);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_KYB3);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_HQC1);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_HQC2);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_HQC3);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K255_KYB512);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K255_HQC128);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K448_KYB1024);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K448_HQC256);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256);\
 	);
 }
 
