@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -257,8 +258,8 @@ static void player_process(MSFilter *f){
 									om = allocb(bytes_pcap, 0);
 									memcpy(om->b_wptr, payload, bytes_pcap);
 									om->b_wptr += bytes_pcap;
-									mblk_set_cseq(om, ntohs(rtph->seq_number));
-									mblk_set_timestamp_info(om, ntohl(rtph->timestamp));
+									mblk_set_cseq(om, rtp_header_get_seqnumber(rtph));
+									mblk_set_timestamp_info(om, rtp_header_get_timestamp(rtph));
 									mblk_set_marker_info(om,markbit);
 								}
 							}
