@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -63,9 +64,15 @@ struct SenderData {
 	int mixer_to_client_extension_id;
 	MSRtpSendRequestMixerToClientDataCb mtc_request_data_cb;
 	void *mtc_request_data_user_data;
+	int mtc_volumes_left_to_send;
+	int mtc_volumes_size;
+	rtp_audio_level_t *mtc_volumes;
 	int client_to_mixer_extension_id;
 	MSRtpSendRequestClientToMixerDataCb ctm_request_data_cb;
 	void *ctm_request_data_user_data;
+	int frame_marking_extension_id;
+	bool_t frame_start;
+	bool_t rtp_transfer_mode;
 };
 
 typedef struct SenderData SenderData;
@@ -141,9 +148,15 @@ static void sender_init(MSFilter * f)
 	d->mixer_to_client_extension_id = 0;
 	d->mtc_request_data_cb = NULL;
 	d->mtc_request_data_user_data = NULL;
+	d->mtc_volumes_left_to_send = 0;
+	d->mtc_volumes_size = 0;
+	d->mtc_volumes = NULL;
 	d->client_to_mixer_extension_id = 0;
 	d->ctm_request_data_cb = NULL;
 	d->ctm_request_data_user_data = NULL;
+	d->frame_marking_extension_id = 0;
+	d->frame_start = TRUE;
+	d->rtp_transfer_mode = FALSE;
 	f->data = d;
 }
 
@@ -261,6 +274,13 @@ static int sender_set_client_to_mixer_data_request_cb(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int sender_set_frame_marking_extension_id(MSFilter *f, void *arg) {
+	SenderData *d = (SenderData *) f->data;
+	int *id = (int *)arg;
+	d->frame_marking_extension_id = *id;
+	return 0;
+}
+
 static int sender_get_sr(MSFilter *f, void *arg){
 	SenderData *d = (SenderData *) f->data;
 	PayloadType *pt;
@@ -442,7 +462,8 @@ static void check_stun_sending(MSFilter *f) {
 	uint64_t last_sent_timeout = 20000;
 	if (rtp_session_get_stats(s)->packet_sent == 0)
 		last_sent_timeout = 2000;
-	if ((d->last_rtp_stun_sent_time == -1) || (  (d->stun_forced_enabled ||  ((f->ticker->time - d->last_sent_time) > last_sent_timeout))
+
+	if ((d->last_rtp_stun_sent_time == -1) || ((d->stun_forced_enabled ||  ((f->ticker->time - d->last_sent_time) > last_sent_timeout))
 											&& (f->ticker->time - d->last_rtp_stun_sent_time) >= 500)) {
 		/* send stun packet every 500 ms: 
 		 * - in absence of any RTP packet for more than last_sent_timeout
@@ -451,7 +472,7 @@ static void check_stun_sending(MSFilter *f) {
 		send_stun_packet(d,TRUE,FALSE);
 	}
 	
-	if ( rtp_session_rtcp_enabled(s) && (d->last_rtcp_stun_sent_time == -1
+	if ( rtp_session_rtcp_enabled(s) && ((d->last_rtcp_stun_sent_time == -1)
 										 || (rtp_session_get_stats(s)->recv_rtcp_packets == 0 /*no need to send stun packets if rtcp packet already received*/
 											 && (f->ticker->time - d->last_rtcp_stun_sent_time) >= 500))) {
 		d->last_rtcp_stun_sent_time = f->ticker->time;
@@ -459,6 +480,55 @@ static void check_stun_sending(MSFilter *f) {
 	}
 }
 
+static mblk_t *create_packet_with_volume_data_at_intervals(MSFilter *f, size_t header_size, const uint8_t *payload, size_t payload_size) {
+	SenderData *d = (SenderData *) f->data;
+	RtpSession *s = d->session;
+	rtp_audio_level_t *audio_levels = NULL;
+	mblk_t *header = NULL;
+
+	if (d->mtc_volumes_left_to_send > 0) {
+		rtp_audio_level_t *tmp = d->mtc_volumes + (d->mtc_volumes_size - d->mtc_volumes_left_to_send);
+
+		if (d->mtc_volumes_left_to_send <= RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL) {
+			header = rtp_session_create_packet_with_mixer_to_client_audio_level(s, 0, d->mixer_to_client_extension_id, d->mtc_volumes_left_to_send, tmp, payload, payload_size);
+			ms_free(d->mtc_volumes);
+			d->mtc_volumes_left_to_send = 0;
+		} else {
+			header = rtp_session_create_packet_with_mixer_to_client_audio_level(s, 0, d->mixer_to_client_extension_id, RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL, tmp, payload, payload_size);
+			d->mtc_volumes_left_to_send -= RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL;
+		}
+
+		return header;
+	}
+
+	if ((d->mixer_to_client_extension_id > 0 || d->client_to_mixer_extension_id > 0)
+		&& f->ticker->time - d->last_audio_level_sent_time > (uint64_t)d->audio_level_send_interval) {
+
+		if (d->mixer_to_client_extension_id > 0 && d->mtc_request_data_cb) {
+			int size = d->mtc_request_data_cb(f, &audio_levels, d->mtc_request_data_user_data);
+			if (audio_levels != NULL) {
+				if (size <= RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL) {
+					header = rtp_session_create_packet_with_mixer_to_client_audio_level(s, 0, d->mixer_to_client_extension_id, size, audio_levels, payload, payload_size);
+					ms_free(audio_levels);
+				} else {
+					header = rtp_session_create_packet_with_mixer_to_client_audio_level(s, 0, d->mixer_to_client_extension_id, RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL, audio_levels, payload, payload_size);
+					d->mtc_volumes = audio_levels;
+					d->mtc_volumes_size = size;
+					d->mtc_volumes_left_to_send = size - RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL;
+				}
+			}
+		} else if (d->client_to_mixer_extension_id > 0 && d->ctm_request_data_cb) {
+			int volume = d->ctm_request_data_cb(f, d->ctm_request_data_user_data);
+			if (!header) header = rtp_session_create_packet(s, 40, payload, payload_size);
+			rtp_add_client_to_mixer_audio_level(header, d->client_to_mixer_extension_id, FALSE, volume);
+		}
+
+		d->last_audio_level_sent_time = f->ticker->time;
+	}
+
+	return header ? header : rtp_session_create_packet(s, 0, payload, payload_size);
+}
+
 static void process_cn(MSFilter *f, SenderData *d, uint32_t timestamp, mblk_t *im ){
 	if (d->cng_data.datasize>0){
 		rtp_header_t *rtp;
@@ -466,7 +536,7 @@ static void process_cn(MSFilter *f, SenderData *d, uint32_t timestamp, mblk_t *i
 		int cn_pt=rtp_profile_find_payload_number(d->session->snd.profile, "CN", 8000, 1);
 
 		/* create the packet, payload type number is the one used for current codec */
-		mblk_t *m=rtp_session_create_packet(d->session, 12, d->cng_data.data, d->cng_data.datasize);
+		mblk_t *m=create_packet_with_volume_data_at_intervals(f, 12, d->cng_data.data, d->cng_data.datasize);
 		mblk_meta_copy(im, m);
 		/* replace payload type in RTP header */
 		rtp=(rtp_header_t*)m->b_rptr;
@@ -495,13 +565,31 @@ static void print_processing_delay_stats(SenderData *d) {
 	}
 }
 
+static void add_frame_marking_extension(SenderData *d, mblk_t *header, mblk_t *im) {
+	uint8_t marker = 0;
+
+	if (d->frame_start) {
+		marker |= RTP_FRAME_MARKER_START;
+		d->frame_start = FALSE;
+	}
+
+	if (mblk_get_marker_info(im)) {
+		marker |= RTP_FRAME_MARKER_END;
+		d->frame_start = TRUE;
+	}
+
+	if (mblk_get_independent_flag(im)) marker |= RTP_FRAME_MARKER_INDEPENDENT;
+	if (mblk_get_discardable_flag(im)) marker |= RTP_FRAME_MARKER_DISCARDABLE;
+
+	rtp_add_frame_marker(header, d->frame_marking_extension_id, marker);
+}
+
 static void _sender_process(MSFilter * f)
 {
 	SenderData *d = (SenderData *) f->data;
 	RtpSession *s = d->session;
 	mblk_t *im;
-	uint32_t timestamp;
-	rtp_audio_level_t audio_levels[RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL] = {{0}};
+	uint32_t timestamp = 0;
 
 	if (d->relay_session_id_size>0 && 
 		( (f->ticker->time-d->last_rsi_time)>5000 || d->last_rsi_time==0) ) {
@@ -514,7 +602,6 @@ static void _sender_process(MSFilter * f)
 	im = ms_queue_get(f->inputs[0]);
 	do {
 		mblk_t *header = NULL;
-
 		timestamp = get_cur_timestamp(f, im);
 		
 		if (d->dtmf != 0 && !d->skip) {
@@ -536,25 +623,16 @@ static void _sender_process(MSFilter * f)
 		}
 		if (im){
 			compute_processing_delay_stats(f, im);
+
+			if (d->rtp_transfer_mode) {
+				rtp_session_sendm_with_ts(s, im, timestamp);
+				continue;
+			}
+
 			if (d->skip == FALSE && d->mute==FALSE){
-				if ((d->mixer_to_client_extension_id > 0 || d->client_to_mixer_extension_id > 0)
-					&& f->ticker->time - d->last_audio_level_sent_time > (uint64_t)d->audio_level_send_interval) {
-
-					if (d->mixer_to_client_extension_id > 0 && d->mtc_request_data_cb) {
-						int size = d->mtc_request_data_cb(f, audio_levels, d->mtc_request_data_user_data);
-						header = rtp_session_create_packet_with_mixer_to_client_audio_level(s, 12, d->mixer_to_client_extension_id, size, audio_levels);
-					}
-					if (d->client_to_mixer_extension_id > 0 && d->ctm_request_data_cb) {
-						int volume = d->ctm_request_data_cb(f, d->ctm_request_data_user_data);
-						if (!header) header = rtp_session_create_packet(s, 12, NULL, 0);
-						rtp_add_client_to_mixer_audio_level(header, d->client_to_mixer_extension_id, FALSE, volume);
-					}
-
-					d->last_audio_level_sent_time = f->ticker->time;
-				}
-
-				if (!header) header = rtp_session_create_packet(s, 12, NULL, 0);
+				header = create_packet_with_volume_data_at_intervals(f, 12, NULL, 0);
 				rtp_set_markbit(header, mblk_get_marker_info(im));
+				if (d->frame_marking_extension_id > 0) add_frame_marking_extension(d, header, im);
 				header->b_cont = im;
 				mblk_meta_copy(im, header);
 				rtp_session_sendm_with_ts(s, header, timestamp);
@@ -570,7 +648,7 @@ static void _sender_process(MSFilter * f)
 			// Send STUN packet as RTP keep alive even if there is no input
 			check_stun_sending(f);
 		}
-	}while ((im = ms_queue_get(f->inputs[0])) != NULL);
+	}while ( (im = ms_queue_get(f->inputs[0]) ) != NULL);
 
 	if (d->last_sent_time == -1) {
 		check_stun_sending(f);
@@ -632,25 +710,43 @@ static int enable_ts_adjustment(MSFilter *f, void *data){
 	return 0;
 }
 
+static int sender_enable_rtp_transfer_mode(MSFilter *f, void *data) {
+	SenderData *d = (SenderData *) f->data;
+	d->rtp_transfer_mode = *(bool_t*)data;
+	return 0;
+}
+
+static int sender_set_active_speaker_ssrc(MSFilter *f, void *data) {
+	SenderData *d = (SenderData *) f->data;
+	uint32_t csrc = *(uint32_t *) data;
+
+	rtp_session_clear_contributing_sources(d->session);
+	if (csrc != 0) rtp_session_add_contributing_source(d->session, csrc, "", NULL, NULL, NULL, NULL, NULL, NULL);
+	return 0;
+}
+
 static MSFilterMethod sender_methods[] = {
-	{MS_RTP_SEND_MUTE, sender_mute},
-	{MS_RTP_SEND_UNMUTE, sender_unmute},
-	{MS_RTP_SEND_SET_SESSION, sender_set_session},
-	{MS_RTP_SEND_SEND_DTMF, sender_send_dtmf},
-	{MS_RTP_SEND_SET_RELAY_SESSION_ID, sender_set_relay_session_id},
-	{MS_RTP_SEND_SET_AUDIO_LEVEL_SEND_INTERVAL, sender_set_audio_level_send_interval},
-	{MS_RTP_SEND_SET_MIXER_TO_CLIENT_EXTENSION_ID, sender_set_mixer_to_client_extension_id},
-	{MS_RTP_SEND_SET_MIXER_TO_CLIENT_DATA_REQUEST_CB, sender_set_mixer_to_client_data_request_cb},
-	{MS_RTP_SEND_SET_CLIENT_TO_MIXER_EXTENSION_ID, sender_set_client_to_mixer_extension_id},
-	{MS_RTP_SEND_SET_CLIENT_TO_MIXER_DATA_REQUEST_CB, sender_set_client_to_mixer_data_request_cb},
-	{MS_FILTER_GET_SAMPLE_RATE, sender_get_sr },
-	{MS_FILTER_GET_NCHANNELS, sender_get_ch },
-	{MS_RTP_SEND_SET_DTMF_DURATION, sender_set_dtmf_duration },
-	{MS_RTP_SEND_SEND_GENERIC_CN, sender_send_generic_cn },
+	{ MS_RTP_SEND_MUTE, sender_mute },
+	{ MS_RTP_SEND_UNMUTE, sender_unmute },
+	{ MS_RTP_SEND_SET_SESSION, sender_set_session },
+	{ MS_RTP_SEND_SEND_DTMF, sender_send_dtmf },
+	{ MS_RTP_SEND_SET_RELAY_SESSION_ID, sender_set_relay_session_id },
+	{ MS_RTP_SEND_SET_AUDIO_LEVEL_SEND_INTERVAL, sender_set_audio_level_send_interval },
+	{ MS_RTP_SEND_SET_MIXER_TO_CLIENT_EXTENSION_ID, sender_set_mixer_to_client_extension_id },
+	{ MS_RTP_SEND_SET_MIXER_TO_CLIENT_DATA_REQUEST_CB, sender_set_mixer_to_client_data_request_cb },
+	{ MS_RTP_SEND_SET_CLIENT_TO_MIXER_EXTENSION_ID, sender_set_client_to_mixer_extension_id },
+	{ MS_RTP_SEND_SET_CLIENT_TO_MIXER_DATA_REQUEST_CB, sender_set_client_to_mixer_data_request_cb },
+	{ MS_RTP_SEND_SET_FRAME_MARKING_EXTENSION_ID, sender_set_frame_marking_extension_id },
+	{ MS_FILTER_GET_SAMPLE_RATE, sender_get_sr },
+	{ MS_FILTER_GET_NCHANNELS, sender_get_ch },
+	{ MS_RTP_SEND_SET_DTMF_DURATION, sender_set_dtmf_duration },
+	{ MS_RTP_SEND_SEND_GENERIC_CN, sender_send_generic_cn },
 	{ MS_RTP_SEND_ENABLE_STUN, sender_enable_stun },
 	{ MS_FILTER_GET_OUTPUT_FMT, get_sender_output_fmt },
 	{ MS_RTP_SEND_ENABLE_TS_ADJUSTMENT, enable_ts_adjustment },
 	{ MS_RTP_SEND_ENABLE_STUN_FORCED, sender_enable_stun_forced },
+	{ MS_RTP_SEND_ENABLE_RTP_TRANSFER_MODE, sender_enable_rtp_transfer_mode },
+	{ MS_RTP_SEND_SET_ACTIVE_SPEAKER_SSRC, sender_set_active_speaker_ssrc },
 	{0, NULL}
 };
 
@@ -699,6 +795,9 @@ struct ReceiverData {
 	bool_t reset_jb;
 	int mixer_to_client_extension_id;
 	int client_to_mixer_extension_id;
+	bool_t rtp_transfer_mode;
+	bool_t csrc_events_enabled;
+	uint32_t last_csrc;
 };
 
 typedef struct ReceiverData ReceiverData;
@@ -709,6 +808,10 @@ static void receiver_init(MSFilter * f)
 	d->session = NULL;
 	d->rate = 8000;
 	d->mixer_to_client_extension_id = 0;
+	d->client_to_mixer_extension_id = 0;
+	d->rtp_transfer_mode = FALSE;
+	d->csrc_events_enabled = FALSE;
+	d->last_csrc = 0;
 	f->data = d;
 }
 
@@ -854,6 +957,20 @@ static void receiver_check_for_extensions(MSFilter *f, mblk_t *m) {
 	}
 }
 
+static void receiver_check_for_csrc_change(MSFilter *f, mblk_t *m) {
+	ReceiverData *d = (ReceiverData *) f->data;
+	uint32_t csrc = 0;
+
+	if (rtp_get_cc(m) > 0) {
+		csrc = rtp_get_csrc(m, 0);
+	}
+
+	if (csrc != d->last_csrc) {
+		ms_filter_notify(f, MS_RTP_RECV_CSRC_CHANGED, &csrc);
+		d->last_csrc = csrc;
+	}
+}
+
 static void receiver_process(MSFilter * f)
 {
 	ReceiverData *d = (ReceiverData *) f->data;
@@ -880,11 +997,17 @@ static void receiver_process(MSFilter * f)
 
 	timestamp = (uint32_t) (f->ticker->time * (d->rate/1000));
 	while ((m = rtp_session_recvm_with_ts(d->session, timestamp)) != NULL) {
+		if (d->rtp_transfer_mode) {
+			ms_queue_put(f->outputs[0], m);
+			continue;
+		}
+
 		if (receiver_check_payload_type(f, d, m)){
 			mblk_set_timestamp_info(m, rtp_get_timestamp(m));
 			mblk_set_marker_info(m, rtp_get_markbit(m));
 			mblk_set_cseq(m, rtp_get_seqnumber(m));
 			receiver_check_for_extensions(f, m);
+			if (d->csrc_events_enabled) receiver_check_for_csrc_change(f, m);
 			rtp_get_payload(m,&m->b_rptr);
 			ms_queue_put(f->outputs[0], m);
 		}else{
@@ -904,11 +1027,25 @@ static int get_receiver_output_fmt(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int receiver_enable_rtp_transfer_mode(MSFilter *f, void *arg) {
+	ReceiverData *d = (ReceiverData *) f->data;
+	d->rtp_transfer_mode = * (bool_t *) arg;
+	return 0;
+}
+
+static int receiver_enable_csrc_events(MSFilter *f, void *arg) {
+	ReceiverData *d = (ReceiverData *) f->data;
+	d->csrc_events_enabled = * (bool_t *) arg;
+	return 0;
+}
+
 static MSFilterMethod receiver_methods[] = {
 	{	MS_RTP_RECV_SET_SESSION	, receiver_set_session	},
 	{	MS_RTP_RECV_RESET_JITTER_BUFFER, receiver_reset_jitter_buffer },
 	{	MS_RTP_RECV_SET_MIXER_TO_CLIENT_EXTENSION_ID, receiver_set_mixer_to_client_extension_id },
 	{	MS_RTP_RECV_SET_CLIENT_TO_MIXER_EXTENSION_ID, receiver_set_client_to_mixer_extension_id },
+	{	MS_RTP_RECV_ENABLE_RTP_TRANSFER_MODE, receiver_enable_rtp_transfer_mode },
+	{	MS_RTP_RECV_ENABLE_CSRC_EVENTS, receiver_enable_csrc_events },
 	{	MS_FILTER_GET_SAMPLE_RATE	, receiver_get_sr		},
 	{	MS_FILTER_GET_NCHANNELS	,	receiver_get_ch	},
 	{ 	MS_FILTER_GET_OUTPUT_FMT, get_receiver_output_fmt },
