@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -45,14 +46,14 @@ enum ImageType {
 #ifdef CHECK_GL_ERROR
 	#define GL_OPERATION(f, x)	\
 		do { \
-		(f->x); \
-		check_GL_errors(f, #x); \
+			(f->x); \
+			check_GL_errors(f, #x); \
 		} while (0);
 
 	#define GL_OPERATION_RET(f, x, ret)	\
 		do { \
-		ret = (f->x); \
-		check_GL_errors(f, #x); \
+			ret = (f->x); \
+			check_GL_errors(f, #x); \
 		} while (0);
 #else
 	#define GL_OPERATION(f, x) (f->x);
@@ -599,13 +600,33 @@ static void ogl_display_render_type(
 		}
 	} else {
 		if (screenW > screenH) {
-			float ratio = gldisp->yuv_size[type].height / (float)(gldisp->yuv_size[type].width);
-			w = screenW * vpw;
-			h = w * ratio;
+			if (gldisp->yuv_size[type].width > gldisp->yuv_size[type].height) {
+				float ratio = gldisp->yuv_size[type].width / (float)(gldisp->yuv_size[type].height);
+				if ((screenW / screenH) > ratio) {
+					w = screenW * vpw;
+					h = w * (gldisp->yuv_size[type].height / (float)(gldisp->yuv_size[type].width));
+				} else {
+					h = screenH * vph;
+					w = h * ratio;
+				}
+			} else {
+				w = screenW * vpw;
+				h = w * (gldisp->yuv_size[type].height / (float)(gldisp->yuv_size[type].width));
+			}
 		} else {
-			float ratio = gldisp->yuv_size[type].width / (float)(gldisp->yuv_size[type].height);
-			h = screenH * vph;
-			w = h * ratio;
+			if (gldisp->yuv_size[type].height > gldisp->yuv_size[type].width) {
+				float ratio = gldisp->yuv_size[type].height / (float)(gldisp->yuv_size[type].width);
+				if ((screenH / screenW) > ratio) {
+					h = screenH * vph;
+					w = h * (gldisp->yuv_size[type].width / (float)(gldisp->yuv_size[type].height));
+				} else {
+					w = screenW * vpw;
+					h = w * ratio;
+				}
+			} else {
+				h = screenH * vph;
+				w = h * (gldisp->yuv_size[type].width / (float)(gldisp->yuv_size[type].height));
+			}
 		}
 	}
 
@@ -623,7 +644,7 @@ static void ogl_display_render_type(
 	GLfloat vTop = gldisp->uvy[type];
 
 	GLfloat arrayBuffer[] = {
-		// Relative coordinates (UV) of the decoded image 
+		// Relative coordinates (UV) of the decoded image
 		uLeft, vTop,
 		uRight, vTop,
 		uLeft, vBottom,
@@ -1416,6 +1437,10 @@ void ogl_display_set_yuv_to_display(struct opengles_display *gldisp, mblk_t *yuv
 	ogl_display_set_yuv(gldisp, yuv, REMOTE_IMAGE);
 }
 
+mblk_t *ogl_display_get_yuv_to_display(struct opengles_display *gldisp){
+	return gldisp->yuv[REMOTE_IMAGE];
+}
+
 void ogl_display_set_preview_yuv_to_display (struct opengles_display *gldisp, mblk_t *yuv) {
 	ogl_display_set_yuv(gldisp, yuv, PREVIEW_IMAGE);
 }
@@ -1442,12 +1467,16 @@ void ogl_display_render (struct opengles_display *gldisp, int orientation, MSVid
 			ms_error("[ogl_display] Failed to make EGLSurface current");
 			render = FALSE;
 		}else{
-			int width, height;// Get current surface size from EGL if we can
+			int width = 0, height = 0;// Get current surface size from EGL if we can
 			if( gldisp->mRenderSurface != EGL_NO_SURFACE
 				&& EGL_TRUE == gldisp->functions->eglQuerySurface(gldisp->mEglDisplay, gldisp->mRenderSurface, EGL_WIDTH, &width)
-				&& EGL_TRUE == gldisp->functions->eglQuerySurface(gldisp->mEglDisplay, gldisp->mRenderSurface, EGL_HEIGHT, &height)
-				&& (width != gldisp->backingWidth || height != gldisp->backingHeight) )// Size has changed : update display (buffers, viewport, etc.)
+				&& EGL_TRUE == gldisp->functions->eglQuerySurface(gldisp->mEglDisplay, gldisp->mRenderSurface, EGL_HEIGHT, &height)){
+				if (width == 0 || height == 0){
+					ms_warning("Is eglQuerySurface() working ? it returned %ix%i.", width, height);
+				}else if (width != gldisp->backingWidth || height != gldisp->backingHeight){ // Size has changed : update display (buffers, viewport, etc.)
 					ogl_display_init(gldisp, f, width, height);
+				}
+			}
 		}
 	}
 	if(render && gldisp->functions->glInitialized && gldisp->shadersLoaded){
