@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -30,12 +31,13 @@ typedef struct SizeConvState{
 	MSVideoSize target_vsize;
 	MSVideoSize in_vsize;
 	YuvBuf outbuf;
+	MSYuvBufAllocator *allocator;
 	MSScalerContext *sws_ctx;
-	mblk_t *om;
 	float fps;
 	float start_time;
 	int frame_count;
 	queue_t rq;
+	bool_t needRefresh;
 } SizeConvState;
 
 
@@ -49,9 +51,10 @@ static void size_conv_init(MSFilter *f){
 	s->in_vsize.width=0;
 	s->in_vsize.height=0;
 	s->sws_ctx=NULL;
-	s->om=NULL;
+	s->allocator = ms_yuv_buf_allocator_new();
 	s->start_time=0;
 	s->frame_count=-1;
+	s->needRefresh=FALSE;
 	s->fps=-1; /* default to process ALL frames */
 	qinit(&s->rq);
 	f->data=s;
@@ -59,6 +62,7 @@ static void size_conv_init(MSFilter *f){
 
 static void size_conv_uninit(MSFilter *f){
 	SizeConvState *s=(SizeConvState*)f->data;
+	ms_yuv_buf_allocator_free(s->allocator);
 	ms_free(s);
 }
 
@@ -68,28 +72,12 @@ static void size_conv_postprocess(MSFilter *f){
 		ms_scaler_context_free(s->sws_ctx);
 		s->sws_ctx=NULL;
 	}
-	if (s->om!=NULL){
-		freemsg(s->om);
-		s->om=NULL;
-	}
 	flushq(&s->rq,0);
 	s->frame_count=-1;
 }
 
-static mblk_t *size_conv_alloc_mblk(SizeConvState *s){
-	if (s->om!=NULL){
-		int ref=dblk_ref_value(s->om->b_datap);
-		if (ref==1){
-			return dupmsg(s->om);
-		}else{
-			/*the last msg is still referenced by somebody else*/
-			ms_message("size_conv_alloc_mblk: Somebody still retaining yuv buffer (ref=%i)",ref);
-			freemsg(s->om);
-			s->om=NULL;
-		}
-	}
-	s->om=ms_yuv_buf_alloc(&s->outbuf,s->target_vsize.width,s->target_vsize.height);
-	return dupmsg(s->om);
+static mblk_t * sizeconv_alloc_mblk(SizeConvState *s){
+	return ms_yuv_buf_allocator_get(s->allocator, &s->outbuf, s->target_vsize.width,s->target_vsize.height);
 }
 
 static MSScalerContext * get_resampler(SizeConvState *s, int w, int h){
@@ -104,6 +92,7 @@ static MSScalerContext * get_resampler(SizeConvState *s, int w, int h){
 			MS_SCALER_METHOD_BILINEAR);
 		s->in_vsize.width=w;
 		s->in_vsize.height=h;
+		ms_message("MSSizeConv: create new scaler context with w %d, h %d", w,h);
 	}
 	return s->sws_ctx;
 }
@@ -150,14 +139,38 @@ static void size_conv_process(MSFilter *f){
 				inbuf.h==s->target_vsize.height){
 				ms_queue_put(f->outputs[0],im);
 			}else{
-				MSScalerContext *sws_ctx=get_resampler(s,inbuf.w,inbuf.h);
-				mblk_t *om=size_conv_alloc_mblk(s);
-				if (ms_scaler_process(sws_ctx,inbuf.planes,inbuf.strides,s->outbuf.planes, s->outbuf.strides)<0){
-					ms_error("MSSizeConv: error in ms_scaler_process().");
-					freemsg(om);
-				}else{
-					mblk_set_timestamp_info(om, mblk_get_timestamp_info(im));
-					ms_queue_put(f->outputs[0],om);
+				int w = s->target_vsize.width;
+				int h = s->target_vsize.height;
+				// keep the same orientation
+				if(ms_video_size_get_orientation((MSVideoSize){inbuf.w, inbuf.h}) != ms_video_size_get_orientation(s->target_vsize)) {
+					s->target_vsize.width = h;
+					s->target_vsize.height = w;
+				}
+				// If input and output do not have the same ratio, this scaler will stretch the image, keep the same aspect ratio to avoid that
+				if (inbuf.w * s->target_vsize.height/s->target_vsize.width != inbuf.h) {
+					if (inbuf.w > inbuf.h) {
+						s->target_vsize.height = inbuf.h * s->target_vsize.width / inbuf.w;
+					} else {
+						s->target_vsize.width = inbuf.w * s->target_vsize.height / inbuf.h;
+					}
+				}
+				if (s->target_vsize.width != w || s->target_vsize.height !=h) {
+					s->needRefresh = TRUE;
+					ms_filter_notify_no_arg(f, MS_FILTER_OUTPUT_FMT_CHANGED);
+				} else if (!s->needRefresh) {
+					MSScalerContext *sws_ctx=get_resampler(s,inbuf.w,inbuf.h);
+					mblk_t *om= sizeconv_alloc_mblk(s);
+					if (om != NULL) {
+						if (ms_scaler_process(sws_ctx,inbuf.planes,inbuf.strides,s->outbuf.planes, s->outbuf.strides)<0){
+							ms_error("MSSizeConv: error in ms_scaler_process().");
+							freemsg(om);
+						}else {
+							mblk_set_timestamp_info(om, mblk_get_timestamp_info(im));
+							ms_queue_put(f->outputs[0],om);
+						}
+					}
+				} else {
+					ms_warning("MSSizeConv: output fmt changed, waiting.");
 				}
 				freemsg(im);
 			}
@@ -176,8 +189,7 @@ static int sizeconv_set_vsize(MSFilter *f, void*arg){
 	SizeConvState *s=(SizeConvState*)f->data;
 	ms_filter_lock(f);
 	s->target_vsize=*(MSVideoSize*)arg;
-	freemsg(s->om);
-	s->om=NULL;
+	ms_message("sizeconv_set_vsize(): set target size w %d, h %d", s->target_vsize.width, s->target_vsize.height);
 	if (s->sws_ctx!=NULL) {
 		ms_scaler_context_free(s->sws_ctx);
 		s->sws_ctx=NULL;
@@ -193,10 +205,18 @@ static int sizeconv_set_fps(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int sizeconv_get_vsize(MSFilter *f, void *data){
+	SizeConvState *s=(SizeConvState*)f->data;
+	MSVideoSize *vsize = (MSVideoSize *)data;
+	vsize->width =  s->target_vsize.width;
+	vsize->height = s->target_vsize.height;
+	return 0;
+}
 
 static MSFilterMethod methods[]={
 	{	MS_FILTER_SET_FPS	,	sizeconv_set_fps	},
 	{	MS_FILTER_SET_VIDEO_SIZE, sizeconv_set_vsize	},
+	{   MS_FILTER_GET_VIDEO_SIZE, sizeconv_get_vsize	},
 	{	0	,	NULL }
 };
 
