@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -129,9 +130,9 @@ static void enc_init(MSFilter *f) {
 	s->frame_count = 0;
 	s->last_fir_seq_nr = -1;
 #ifdef PICTURE_ID_ON_16_BITS
-	s->picture_id = (ortp_random() & 0x7FFF) | 0x8000;
+	s->picture_id = (bctbx_random() & 0x7FFF) | 0x8000;
 #else
-	s->picture_id = ortp_random() & 0x007F;
+	s->picture_id = bctbx_random() & 0x007F;
 #endif
 	s->avpf_enabled = FALSE;
 	enc_reset_frames_state(s);
@@ -196,7 +197,6 @@ static void enc_init_impl(MSFilter *f){
 	s->cfg.g_error_resilient = VPX_ERROR_RESILIENT_DEFAULT|VPX_ERROR_RESILIENT_PARTITIONS;
 	s->cfg.g_lag_in_frames = 0;
 
-
 #if defined(__ANDROID__) || (TARGET_OS_IPHONE == 1) || defined(__arm__) || defined(_M_ARM)
 	cpuused = 10 - s->cfg.g_threads; /*cpu/quality tradeoff: positive values decrease CPU usage at the expense of quality*/
 	if (cpuused < 7) cpuused = 7; /*values beneath 7 consume too much CPU*/
@@ -234,7 +234,7 @@ static void enc_init_impl(MSFilter *f){
 
 static void enc_preprocess(MSFilter *f) {
 	EncState *s = (EncState *)f->data;
-	
+
 	enc_init_impl(f);
 	s->invalid_frame_reported = FALSE;
 	vp8rtpfmt_packer_init(&s->packer);
@@ -244,7 +244,7 @@ static void enc_preprocess(MSFilter *f) {
 		ms_video_starter_init(&s->starter);
 	}
 
-	s->process_thread = ms_worker_thread_new();
+	s->process_thread = ms_worker_thread_new("MSVp8Enc");
 	qinit(&s->entry_q);
 	s->exit_q = ms_queue_new(0, 0, 0, 0);
 	s->ready = TRUE;
@@ -476,7 +476,7 @@ static void enc_process_frame_task(void *obj) {
 	if (!im) {
 		im = prev_im;
 	}
-	
+
 	ms_filter_unlock(f);
 
 	if (skipped_count > 0){
@@ -520,6 +520,7 @@ static void enc_process_frame_task(void *obj) {
 		const vpx_codec_cx_pkt_t *pkt;
 		bctbx_list_t *list = NULL;
 		int current_partition_id = -1;
+		bool_t is_independent_frame = FALSE;
 		
 		/* Update the frames state. */
 		is_ref_frame=FALSE;
@@ -543,6 +544,7 @@ static void enc_process_frame_task(void *obj) {
 		}
 		if (is_frame_independent(flags)){
 			s->frames_state.last_independent_frame=s->frame_count;
+			is_independent_frame = TRUE;
 		}
 
 		/* Pack the encoded frame. */
@@ -554,6 +556,7 @@ static void enc_process_frame_task(void *obj) {
 				memcpy(packet->m->b_wptr, pkt->data.frame.buf, pkt->data.frame.sz);
 				packet->m->b_wptr += pkt->data.frame.sz;
 				mblk_set_timestamp_info(packet->m, mblk_get_timestamp_info(im));
+				mblk_set_independent_flag(packet->m, is_independent_frame);
 				packet->pd = ms_new0(Vp8RtpFmtPayloadDescriptor, 1);
 				packet->pd->non_reference_frame = s->avpf_enabled && !is_ref_frame;
 				if (s->avpf_enabled == TRUE) {
@@ -622,6 +625,7 @@ static void enc_process(MSFilter *f) {
 	EncState *s = (EncState *)f->data;
 	mblk_t *entry_f;
 	mblk_t *exit_f;
+	MSPicture yuv;
 
 	ms_filter_lock(f);
 
@@ -637,9 +641,15 @@ static void enc_process(MSFilter *f) {
 
 	/* If we have a frame then we put it in the entry queue and we call the task enc_process_frame_task in async */
 	if ((entry_f = ms_queue_peek_last(f->inputs[0])) != NULL) {
+		ms_yuv_buf_init_from_mblk(&yuv, entry_f);
+		if (yuv.w != s->vconf.vsize.width || yuv.h != s->vconf.vsize.height)
+			ms_error("enc_process: get yuv w %d, h %d and vconf w %d, h %d", yuv.w, yuv.h, s->vconf.vsize.width, s->vconf.vsize.height);
+		
 		ms_queue_remove(f->inputs[0], entry_f);
 		putq(&s->entry_q, entry_f);
+		ms_mutex_lock(&s->vp8_mutex);
 		ms_worker_thread_add_task(s->process_thread, enc_process_frame_task, (void*)f);
+		ms_mutex_unlock(&s->vp8_mutex);
 	}
 
 	/* Put each frame we have in exit_q in f->output[0] */
@@ -654,6 +664,7 @@ static void enc_process(MSFilter *f) {
 static void enc_postprocess(MSFilter *f) {
 	EncState *s = (EncState *)f->data;
 	ms_worker_thread_destroy(s->process_thread, FALSE);
+	s->process_thread = NULL;
 	if (s->ready) vpx_codec_destroy(&s->codec);
 	vp8rtpfmt_packer_uninit(&s->packer);
 	flushq(&s->entry_q,0);
@@ -910,6 +921,9 @@ typedef struct DecState {
 	unsigned int yuv_width, yuv_height;
 	MSAverageFPS fps;
 	uint32_t last_timestamp;
+	int max_threads;
+	int discarded_frames_count;
+	int output_frames_count;
 	bool_t first_image_decoded;
 	bool_t avpf_enabled;
 	bool_t freeze_on_error;
@@ -938,6 +952,18 @@ static void dec_init(MSFilter *f) {
 	s->first_image_decoded = FALSE;
 	s->avpf_enabled = FALSE;
 	s->freeze_on_error = TRUE;
+#if TARGET_OS_OSX
+	/**
+	 * On macosx 10.14 realtime processing is not ensured on dual core machine with ms_factory_get_cpu_count() <= 4.
+	 * Value belows is a tradeoff between scalability and realtime.
+	 * The loss of performance is correlated to the number of threads allocated for the decoding.
+	 * It appears to be due to the inefficiency of spinlocks used by the libvpx.
+	 */
+	s->max_threads = MAX(ms_factory_get_cpu_count(f->factory)-2,1);
+#else
+	s->max_threads = ms_factory_get_cpu_count(f->factory);
+#endif
+	
 	ms_cond_init(&s->thread_cond, NULL);
 	ms_queue_init(&s->entry_q);
 	ms_queue_init(&s->exit_q);
@@ -950,12 +976,7 @@ static int dec_initialize_impl(MSFilter *f){
 	vpx_codec_dec_cfg_t cfg;
 
 	memset(&cfg, 0, sizeof(cfg));
-#if TARGET_OS_OSX
-	//On macosx 10.14 realtime processing is not ensured on dual core machine with ms_factory_get_cpu_count() <= 4. Value belows is a tradeoff between scalability and realtime
-	cfg.threads = MAX(ms_factory_get_cpu_count(f->factory)-2,1);
-#else
-	cfg.threads = ms_factory_get_cpu_count(f->factory);
-#endif
+	cfg.threads = s->max_threads;
 	if (vpx_codec_dec_init(&s->codec, s->iface, &cfg, s->flags)){
 		ms_error("Failed to initialize VP8 decoder");
 		return -1;
@@ -983,12 +1004,13 @@ static void dec_preprocess(MSFilter* f) {
 		}
 
 		if (dec_initialize_impl(f) != 0) return;
-		ms_message("VP8: initializing decoder context: avpf=[%i] freeze_on_error=[%i]",s->avpf_enabled,s->freeze_on_error);
+		ms_message("VP8: initializing decoder context: avpf=[%i] freeze_on_error=[%i] max_threads=[%i]",
+			   s->avpf_enabled, s->freeze_on_error, s->max_threads);
 		vp8rtpfmt_unpacker_init(&s->unpacker, f, s->avpf_enabled, s->freeze_on_error, (s->flags & VPX_CODEC_USE_INPUT_FRAGMENTS) ? TRUE : FALSE);
 		s->first_image_decoded = FALSE;
 		s->ready=TRUE;
 	}
-	
+
 	s->thread_running = TRUE;
 	s->waiting = FALSE;
 	ms_thread_create(&s->thread, NULL, dec_processing_thread, f);
@@ -1022,17 +1044,17 @@ static void *dec_processing_thread(void *obj) {
 			s->waiting = FALSE;
 			continue;
 		}
-		
+
 		/* Unpack RTP payload format for VP8. */
 		vp8rtpfmt_unpacker_feed(&s->unpacker, &s->entry_q);
 		ms_filter_unlock(f);
-	
+
 		/* Decode unpacked VP8 frames. */
 		while (vp8rtpfmt_unpacker_get_frame(&s->unpacker, &frame, &frame_info) == 0) {
 			vpx_codec_err_t err;
 			vpx_image_t *img;
 			vpx_codec_iter_t iter = NULL;
-			
+
 			while ((im = ms_queue_get(&frame)) != NULL) {
 				err = vpx_codec_decode(&s->codec, im->b_rptr, (unsigned int)(im->b_wptr - im->b_rptr), NULL, 0);
 				if ((s->flags & VPX_CODEC_USE_INPUT_FRAGMENTS) && mblk_get_marker_info(im)) {
@@ -1044,43 +1066,46 @@ static void *dec_processing_thread(void *obj) {
 				s->last_timestamp = mblk_get_timestamp_info(im);
 				freemsg(im);
 			}
-	
+
 			/* Get decoded frame */
 			while ((img = vpx_codec_get_frame(&s->codec, &iter))) {
 				int i, j;
 				int reference_updates = 0;
 				mblk_t *yuv_msg;
-	
+
 				if (vpx_codec_control(&s->codec, VP8D_GET_LAST_REF_UPDATES, &reference_updates) == 0) {
 					if (frame_info.pictureid_present && ((reference_updates & VP8_GOLD_FRAME) || (reference_updates & VP8_ALTR_FRAME))) {
 						vp8rtpfmt_send_rpsi(&s->unpacker, frame_info.pictureid);
 					}
 				}
-	
+
 				if (s->yuv_width != img->d_w || s->yuv_height != img->d_h) {
 					ms_message("MSVp8Dec: video is %ix%i", img->d_w, img->d_h);
 					s->yuv_width = img->d_w;
 					s->yuv_height = img->d_h;
 					ms_filter_notify_no_arg(f, MS_FILTER_OUTPUT_FMT_CHANGED);
 				}
-	
+
 				yuv_msg = ms_yuv_buf_allocator_get(s->allocator, &s->outbuf, img->d_w, img->d_h);
-				
+
 				ms_average_fps_update(&s->fps, (uint32_t)f->ticker->time);
 				if (!s->first_image_decoded) {
 					s->first_image_decoded = TRUE;
 					ms_filter_notify_no_arg(f, MS_VIDEO_DECODER_FIRST_IMAGE_DECODED);
 				}
-				
+
 				if (yuv_msg == NULL) {
-					ms_warning("MSVp8Dec: no more output buffers, frame is discarded.");
+					s->discarded_frames_count++;
+					ms_warning("%s[%p]: no more output buffers, [%i] discarded frames, [%i] output frames", f->desc->name, f,
+						s->discarded_frames_count, s->output_frames_count);
 				}else{
+					s->output_frames_count++;
 					/* copy frame to destination mblk_t */
 					for (i = 0; i < 3; i++) {
 						uint8_t *dest = s->outbuf.planes[i];
 						uint8_t *src = img->planes[i];
 						int h = img->d_h >> ((i > 0) ? 1 : 0);
-		
+
 						for (j = 0; j < h; j++) {
 							memcpy(dest, src, s->outbuf.strides[i]);
 							dest += s->outbuf.strides[i];
@@ -1117,7 +1142,7 @@ static void dec_process(MSFilter *f) {
 		ms_filter_unlock(f);
 		return;
 	}
-	
+
 	if (f->ticker->time % 1000 == 0){
 		ms_message("VP8 dec: entry queue size: %i packets.", (int)s->entry_q.q.q_mcount);
 	}
@@ -1206,11 +1231,18 @@ static int dec_get_out_fmt(MSFilter *f, void *data){
 	return 0;
 }
 
+static int dec_set_max_threads(MSFilter *f, void *data){
+	DecState *s = (DecState *)f->data;
+	s->max_threads = *(int*)data;
+	return 0;
+}
+
 static MSFilterMethod dec_methods[] = {
 	{ MS_VIDEO_DECODER_RESET_FIRST_IMAGE_NOTIFICATION, dec_reset_first_image },
 	{ MS_VIDEO_DECODER_ENABLE_AVPF,                    dec_enable_avpf       },
 	{ MS_VIDEO_DECODER_FREEZE_ON_ERROR,                dec_freeze_on_error   },
 	{ MS_VIDEO_DECODER_RESET,                          dec_reset             },
+	{ MS_VIDEO_DECODER_SET_MAX_THREADS,                dec_set_max_threads   },
 	{ MS_FILTER_GET_VIDEO_SIZE,                        dec_get_vsize         },
 	{ MS_FILTER_GET_FPS,                               dec_get_fps           },
 	{ MS_FILTER_GET_OUTPUT_FMT,                        dec_get_out_fmt       },
