@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -877,18 +878,36 @@ void audio_stream_set_is_speaking_callback (AudioStream *s, AudioStreamIsSpeakin
 	s->user_pointer=user_pointer;
 }
 
+void audio_stream_set_is_muted_callback (AudioStream *s, AudioStreamIsMutedCallback cb, void *user_pointer){
+	s->is_muted_cb=cb;
+	s->user_pointer=user_pointer;
+}
+
 static void on_volumes_received(void *data, MSFilter *f, unsigned int event_id, void *event_arg) {
 	AudioStream *as=(AudioStream*)data;
-	rtp_audio_level_t *mtc_volumes;
-	int i;
+	rtp_audio_level_t *volumes = (rtp_audio_level_t *)event_arg;
+	int new_volume;
 
 	switch(event_id) {
 		case MS_RTP_RECV_MIXER_TO_CLIENT_AUDIO_LEVEL_RECEIVED:
-			mtc_volumes = (rtp_audio_level_t *) event_arg;
-			audio_stream_volumes_reset_values(as->participants_volumes);
-			for(i = 0; i < RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL && mtc_volumes[i].csrc != 0; i++) {
-				audio_stream_volumes_insert(as->participants_volumes, mtc_volumes[i].csrc, (int)ms_volume_dbov_to_dbm0(mtc_volumes[i].dbov));
+			for(int i = 0; i < RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL && volumes[i].csrc != 0; i++) {
+				new_volume = (int)ms_volume_dbov_to_dbm0(volumes[i].dbov);
+
+				if (as->is_muted_cb) {
+					int volume = audio_stream_volumes_find(as->participants_volumes, volumes[i].csrc);
+					if (volume == AUDIOSTREAMVOLUMES_NOT_FOUND && new_volume == MS_VOLUME_DB_MUTED) {
+						// Notify if the first volume we receive is a mute
+						as->is_muted_cb(as->user_pointer, volumes[i].csrc, TRUE);
+					} else if ((volume == MS_VOLUME_DB_MUTED && new_volume != MS_VOLUME_DB_MUTED)
+						|| (volume != MS_VOLUME_DB_MUTED && new_volume == MS_VOLUME_DB_MUTED)) {
+						// Otherwise notify if the participant mutes or unmutes himself
+						as->is_muted_cb(as->user_pointer, volumes[i].csrc, new_volume == MS_VOLUME_DB_MUTED);
+					}
+				}
+
+				audio_stream_volumes_insert(as->participants_volumes, volumes[i].csrc, new_volume);
 			}
+
 			if (!as->is_speaking_cb)
 				break;
 
@@ -912,7 +931,21 @@ static void on_volumes_received(void *data, MSFilter *f, unsigned int event_id,
 
 			break;
 		case MS_RTP_RECV_CLIENT_TO_MIXER_AUDIO_LEVEL_RECEIVED:
-			// Do nothing for now
+			new_volume = (int)ms_volume_dbov_to_dbm0(volumes->dbov);
+
+			if (as->is_muted_cb) {
+				int volume = audio_stream_volumes_find(as->participants_volumes, volumes->csrc);
+				if (volume == AUDIOSTREAMVOLUMES_NOT_FOUND && new_volume == MS_VOLUME_DB_MUTED) {
+					// Notify if the first volume we receive is a mute
+					as->is_muted_cb(as->user_pointer, volumes->csrc, TRUE);
+				} else if ((volume == MS_VOLUME_DB_MUTED && new_volume != MS_VOLUME_DB_MUTED)
+					|| (volume != MS_VOLUME_DB_MUTED && new_volume == MS_VOLUME_DB_MUTED)) {
+					// Otherwise notify if the participant mutes or unmutes himself
+					as->is_muted_cb(as->user_pointer, volumes->csrc, new_volume == MS_VOLUME_DB_MUTED);
+				}
+			}
+
+			audio_stream_volumes_insert(as->participants_volumes, volumes->csrc, new_volume);
 			break;
 	}
 }
@@ -920,11 +953,14 @@ static void on_volumes_received(void *data, MSFilter *f, unsigned int event_id,
 static int request_stream_volume(MSFilter *filter, void* user_data) {
 	AudioStream *as = (AudioStream *)user_data;
 	MSFilter *volume_filter = as->volsend;
-	float db = MS_VOLUME_DB_LOWEST;
-	
-	ms_filter_call_method(volume_filter, MS_VOLUME_GET, &db);
+	float ret;
 
-	return ms_volume_dbm0_to_dbov(db);
+	ms_filter_call_method(volume_filter, MS_VOLUME_GET_GAIN, &ret);
+	if (ret == 0)
+		return -127;
+
+	ms_filter_call_method(volume_filter, MS_VOLUME_GET, &ret);
+	return ms_volume_dbm0_to_dbov(ret);
 }
 
 void audio_stream_set_audio_route_changed_callback (AudioStream *s, MSAudioRouteChangedCallback cb, void *audio_route_changed_cb_user_data) {
@@ -1201,7 +1237,11 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 		ms_filter_call_method(stream->dtmfgen_rtp,MS_FILTER_SET_NCHANNELS,&nchannels);
 	}
 
-	ms_filter_call_method(stream->soundread, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
+	// Do not set sample rate if the input is a file
+	if (io->input.file == NULL) {
+		ms_filter_call_method(stream->soundread, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
+	}
+
 	ms_filter_call_method(stream->soundread, MS_FILTER_SET_NCHANNELS, &nchannels);
 	if (ms_filter_has_method(stream->soundread, MS_AUDIO_CAPTURE_ENABLE_AEC)) {
 		bool_t aec_enabled = TRUE;
@@ -1558,6 +1598,8 @@ void audio_stream_play(AudioStream *st, const char *name){
 			if (st->read_resampler){
 				audio_stream_configure_resampler(st, st->read_resampler,st->soundread,st->ms.encoder);
 			}
+			int pause_time = 500;
+			ms_filter_call_method(st->soundread,MS_PLAYER_SET_LOOP,&pause_time);
 			ms_filter_call_method_noarg(st->soundread,MS_FILE_PLAYER_START);
 		}
 	}else{
@@ -1581,8 +1623,7 @@ void audio_stream_record(AudioStream *st, const char *name){
 	}
 }
 
-
-int audio_stream_mixed_record_open(AudioStream *st, const char* filename){
+int audio_stream_set_mixed_record_file(AudioStream *st, const char* filename){
 	if (!(st->features & AUDIO_STREAM_FEATURE_MIXED_RECORDING)){
 		if (audio_stream_started(st)){
 			ms_error("Too late - you cannot request a mixed recording when the stream is running because it did not have AUDIO_STREAM_FEATURE_MIXED_RECORDING feature.");
@@ -1711,6 +1752,7 @@ AudioStream *audio_stream_new_with_sessions(MSFactory *factory, const MSMediaStr
 
 	stream->participants_volumes = audio_stream_volumes_new();
 	stream->mixer_to_client_extension_id = 0;
+	stream->client_to_mixer_extension_id = 0;
 
 	return stream;
 }
@@ -1937,13 +1979,9 @@ void audio_stream_stop(AudioStream * stream){
 			stream->ms.state=MSStreamStopped;
 			ms_ticker_detach(stream->ms.sessions.ticker,stream->soundread);
 			ms_ticker_detach(stream->ms.sessions.ticker,stream->ms.rtprecv);
-
-			if (stream->ms.ice_check_list != NULL) {
-				ice_check_list_print_route(stream->ms.ice_check_list, "Audio session's route");
-				stream->ms.ice_check_list = NULL;
-			}
-			rtp_stats_display(rtp_session_get_stats(stream->ms.sessions.rtp_session),
-				"             AUDIO SESSION'S RTP STATISTICS                ");
+			
+			ms_message("Stopping AudioStream.");
+			media_stream_print_summary(&stream->ms);
 
 			/*dismantle the outgoing graph*/
 			ms_connection_helper_start(&h);
@@ -2228,3 +2266,11 @@ void audio_stream_set_client_to_mixer_extension_id(AudioStream *stream, int exte
 int audio_stream_get_participant_volume(const AudioStream *stream, uint32_t participant_ssrc) {
 	return audio_stream_volumes_find(stream->participants_volumes, participant_ssrc);
 }
+
+uint32_t audio_stream_get_send_ssrc(const AudioStream *stream) {
+	return rtp_session_get_send_ssrc(stream->ms.sessions.rtp_session);
+}
+
+uint32_t audio_stream_get_recv_ssrc(const AudioStream *stream) {
+	return rtp_session_get_recv_ssrc(stream->ms.sessions.rtp_session);
+}
