@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -119,6 +120,7 @@ void media_stream_init(MediaStream *stream, MSFactory *factory, const MSMediaStr
 		ms_dtls_srtp_set_stream_sessions(sessions->dtls_context, &stream->sessions);
 	}
 	media_stream_add_tmmbr_handler(stream, media_stream_tmmbr_received, stream);
+	stream->stun_allowed = TRUE;
 }
 
 void media_stream_add_tmmbr_handler(MediaStream *stream, void (*on_tmmbr_received)(const OrtpEventData *evd, void *), void *user_data){
@@ -139,7 +141,7 @@ void media_stream_remove_tmmbr_handler(MediaStream *stream, void (*on_tmmbr_rece
 RtpSession * ms_create_duplex_rtp_session(const char* local_ip, int loc_rtp_port, int loc_rtcp_port, int mtu) {
 	RtpSession *rtpr;
 	const int socket_buf_size=2000000;
-	
+
 	rtpr = rtp_session_new(RTP_SESSION_SENDRECV);
 	rtp_session_set_recv_buf_size(rtpr, MAX(mtu , MS_MINIMAL_MTU));
 	rtp_session_set_scheduling_mode(rtpr, 0);
@@ -165,7 +167,7 @@ RtpSession * ms_create_duplex_rtp_session(const char* local_ip, int loc_rtp_port
 	rtp_session_set_send_ts_offset(rtpr, (uint32_t)bctbx_random());
 	rtp_session_enable_avpf_feature(rtpr, ORTP_AVPF_FEATURE_TMMBR, TRUE);
 	disable_checksums(rtp_session_get_rtp_socket(rtpr));
-	
+
 	/* Enlarge kernel socket buffers, which is necessary for video streams because large amounts of data can arrive between
 	 * two ticks of mediastreamer2 processing.
 	 * Previously, this was done only for VideoStreams, but since the sockets in the audio RtpSession may be used for video too
@@ -196,18 +198,12 @@ const char * media_stream_type_str(MediaStream *stream) {
 	return ms_format_type_to_string(stream->type);
 }
 
-void media_stream_reset_zrtp_context(MediaStream *stream){
-	MSMediaStreamSessions *sessions = &(stream->sessions);
-	if (sessions && sessions->zrtp_context!=NULL) {
-		ms_zrtp_context_reset(sessions->zrtp_context);
-	}
-}
-
 void ms_media_stream_sessions_uninit(MSMediaStreamSessions *sessions){
 	if (sessions->srtp_context) {
 		ms_srtp_context_delete(sessions->srtp_context);
 		sessions->srtp_context=NULL;
 	}
+
 	if (sessions->rtp_session) {
 		rtp_session_destroy(sessions->rtp_session);
 		sessions->rtp_session=NULL;
@@ -228,6 +224,7 @@ void ms_media_stream_sessions_uninit(MSMediaStreamSessions *sessions){
 
 void media_stream_free(MediaStream *stream) {
 	media_stream_remove_tmmbr_handler(stream, media_stream_tmmbr_received, stream);
+
 	if (stream->sessions.zrtp_context != NULL) {
 		ms_zrtp_set_stream_sessions(stream->sessions.zrtp_context, NULL);
 	}
@@ -289,10 +286,14 @@ void media_stream_enable_adaptive_jittcomp(MediaStream *stream, bool_t enabled)
 	rtp_session_enable_adaptive_jitter_compensation(stream->sessions.rtp_session, enabled);
 }
 
+void media_stream_set_stun_allowed(MediaStream *stream, bool_t value){
+	stream->stun_allowed = value;
+
+}
 
 /* This function decides whether it is necessary to send dummy stun packets for firewall opening. */
 static void media_stream_configure_stun_packet_sending(MediaStream *stream){
-	bool_t stun_enabled = TRUE;
+	bool_t stun_enabled = stream->stun_allowed;
 	if (stream->ice_check_list) stun_enabled = FALSE;
 	if (stream->sessions.rtp_session->bundle && !stream->sessions.rtp_session->is_primary){
 		stun_enabled = FALSE;
@@ -301,8 +302,9 @@ static void media_stream_configure_stun_packet_sending(MediaStream *stream){
 		ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_ENABLE_STUN, &stun_enabled);
 		if (stream->sessions.dtls_context) {
 			/* In case STUN packets are necessary, and DTLS is used and encryption mandatory is set to TRUE,
-			 * no packets will be emitted at all. We must configure the rtpsend filter to regularly send dummy stun packets
-			 * to ensure that firewall gets open to the remote endpoint. 
+			 * no RTP packets will be emitted at all, until handshake takes places.
+			 * We must configure the rtpsend filter to regularly send dummy stun packets
+			 * to ensure that firewall gets open to the remote endpoint.
 			 * Note that we are unable to check here if encryption mandatory is on (due to order of operation that might be
 			 * undetermined), but it is acceptable to send dummy stun packets even if encryption mandatory is off.
 			 */
@@ -317,7 +319,7 @@ void media_stream_enable_dtls(MediaStream *stream, const MSDtlsSrtpParams *param
 		MSDtlsSrtpParams params_copy = *params;
 		ms_message("Start DTLS media stream context in stream session [%p]", &(stream->sessions));
 		if (params_copy.mtu == 0) params_copy.mtu = ms_factory_get_mtu(stream->factory);
-		
+
 		stream->sessions.dtls_context=ms_dtls_srtp_context_new(&(stream->sessions), &params_copy);
 		media_stream_configure_stun_packet_sending(stream);
 	}
@@ -364,7 +366,7 @@ bool_t ms_is_ipv6(const char *remote) {
 }
 
 bool_t ms_is_multicast_addr(const struct sockaddr *addr) {
-	return ortp_is_multicast_addr(addr);
+	return bctbx_is_multicast_addr(addr);
 }
 
 bool_t ms_is_multicast(const char *address) {
@@ -399,6 +401,9 @@ static void media_stream_process_rtcp(MediaStream *stream, mblk_t *m, time_t cur
 
 void media_stream_set_direction(MediaStream *stream, MediaStreamDir dir) {
 	stream->direction = dir;
+	if (dir == MediaStreamSendOnly && stream->bandwidth_controller) {
+		ms_bandwidth_controller_elect_controlled_streams(stream->bandwidth_controller);
+	}
 }
 
 MediaStreamDir media_stream_get_direction(const MediaStream *stream) {
@@ -419,7 +424,7 @@ void media_stream_iterate(MediaStream *stream){
 			if (stream->direction != MediaStreamSendOnly) { // Local quality indicator in send only will be wrong
 				ms_quality_indicator_update_local(stream->qi);
 			}
-		} 
+		}
 	}
 	stream->last_iterate_time=curtime;
 
@@ -590,7 +595,6 @@ MSCryptoSuite ms_crypto_suite_build_from_name_params(const MSCryptoSuiteNamePara
 		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) return MS_AES_128_SHA1_32_NO_AUTH;
 		else return MS_AES_128_SHA1_32;
 	}else if ( keywordcmp ("AES_256_CM_HMAC_SHA1_32", name) == 0 ){
-
 		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
 		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
 		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
@@ -608,6 +612,18 @@ MSCryptoSuite ms_crypto_suite_build_from_name_params(const MSCryptoSuiteNamePara
 		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
 		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
 		else return MS_AES_CM_256_SHA1_80;
+	}else if ( keywordcmp ("AEAD_AES_128_GCM", name) == 0 ){
+		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
+		else return MS_AEAD_AES_128_GCM;
+	}else if ( keywordcmp ("AEAD_AES_256_GCM", name) == 0 ){
+		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
+		else return MS_AEAD_AES_256_GCM;
 	}
 error:
 	ms_error("Unsupported crypto suite '%s' with parameters '%s'",name, parameters ? parameters : "");
@@ -623,49 +639,55 @@ bool_t ms_crypto_suite_is_unauthenticated(MSCryptoSuite cs){
 }
 
 int ms_crypto_suite_to_name_params(MSCryptoSuite cs, MSCryptoSuiteNameParams *params ){
-     params->name=NULL;
-     params->params=NULL;
-     switch(cs){
-        case MS_CRYPTO_SUITE_INVALID:
-     		break;
-        case MS_AES_128_SHA1_80:
-     		params->name= "AES_CM_128_HMAC_SHA1_80";
-     		break;
-     	case MS_AES_128_SHA1_32:
-     		params->name="AES_CM_128_HMAC_SHA1_32";
-     		break;
-     	case MS_AES_128_SHA1_80_NO_AUTH:
-     		params->name="AES_CM_128_HMAC_SHA1_80";
-     		params->params="UNAUTHENTICATED_SRTP";
-     		break;
-     	case MS_AES_128_SHA1_32_NO_AUTH:
-     		params->name="AES_CM_128_HMAC_SHA1_32";
-     		params->params="UNAUTHENTICATED_SRTP";
-     		break;
-     	case MS_AES_128_SHA1_80_SRTP_NO_CIPHER:
-     		params->name="AES_CM_128_HMAC_SHA1_80";
-     		params->params="UNENCRYPTED_SRTP";
-     		break;
-     	case MS_AES_128_SHA1_80_SRTCP_NO_CIPHER:
-     		params->name="AES_CM_128_HMAC_SHA1_80";
-     		params->params="UNENCRYPTED_SRTCP";
-     		break;
-     	case MS_AES_128_SHA1_80_NO_CIPHER:
-     		params->name="AES_CM_128_HMAC_SHA1_80";
-     		params->params="UNENCRYPTED_SRTP UNENCRYPTED_SRTCP";
-     		break;
-     	case MS_AES_256_SHA1_80:
-     		params->name="AES_256_CM_HMAC_SHA1_80";
-     		break;
-     	case MS_AES_CM_256_SHA1_80:
-     	    params->name="AES_CM_256_HMAC_SHA1_80";
-     	    break;
-     	case MS_AES_256_SHA1_32:
-     		params->name= "AES_256_CM_HMAC_SHA1_32";
-     		break;
-     }
-     if (params->name==NULL) return -1;
-     return 0;
+	params->name=NULL;
+	params->params=NULL;
+	switch(cs){
+		case MS_CRYPTO_SUITE_INVALID:
+			break;
+		case MS_AES_128_SHA1_80:
+			params->name= "AES_CM_128_HMAC_SHA1_80";
+			break;
+		case MS_AES_128_SHA1_32:
+			params->name="AES_CM_128_HMAC_SHA1_32";
+			break;
+		case MS_AES_128_SHA1_80_NO_AUTH:
+			params->name="AES_CM_128_HMAC_SHA1_80";
+			params->params="UNAUTHENTICATED_SRTP";
+			break;
+		case MS_AES_128_SHA1_32_NO_AUTH:
+			params->name="AES_CM_128_HMAC_SHA1_32";
+			params->params="UNAUTHENTICATED_SRTP";
+			break;
+		case MS_AES_128_SHA1_80_SRTP_NO_CIPHER:
+			params->name="AES_CM_128_HMAC_SHA1_80";
+			params->params="UNENCRYPTED_SRTP";
+			break;
+		case MS_AES_128_SHA1_80_SRTCP_NO_CIPHER:
+			params->name="AES_CM_128_HMAC_SHA1_80";
+			params->params="UNENCRYPTED_SRTCP";
+			break;
+		case MS_AES_128_SHA1_80_NO_CIPHER:
+			params->name="AES_CM_128_HMAC_SHA1_80";
+			params->params="UNENCRYPTED_SRTP UNENCRYPTED_SRTCP";
+			break;
+		case MS_AES_256_SHA1_80:
+			params->name="AES_256_CM_HMAC_SHA1_80";
+			break;
+		case MS_AES_CM_256_SHA1_80:
+			params->name="AES_CM_256_HMAC_SHA1_80";
+			break;
+		case MS_AES_256_SHA1_32:
+			params->name= "AES_256_CM_HMAC_SHA1_32";
+			break;
+		case MS_AEAD_AES_128_GCM:
+			params->name= "AEAD_AES_128_GCM";
+			break;
+		case MS_AEAD_AES_256_GCM:
+			params->name= "AEAD_AES_256_GCM";
+			break;
+	}
+	if (params->name==NULL) return -1;
+	return 0;
 }
 
 OrtpEvDispatcher* media_stream_get_event_dispatcher(const MediaStream *stream) {
@@ -688,6 +710,8 @@ const char *ms_resource_type_to_string(MSResourceType type){
 			return "MSResourceSoundcard";
 		case MSResourceVoid:
 			return "MSResourceVoid";
+		case MSResourceItc:
+			return "MSResourceItc";
 	}
 	return "INVALID";
 }
@@ -706,6 +730,7 @@ bool_t ms_media_resource_is_consistent(const MSMediaResource *r){
 		case MSResourceFile:
 			/*setting up file player/recorder without specifying the file to play immediately is allowed*/
 		case MSResourceDefault:
+		case MSResourceItc:
 			return TRUE;
 		case MSResourceInvalid:
 			ms_error("Invalid resource type specified");
@@ -821,3 +846,23 @@ void media_stream_tmmbr_received(const OrtpEventData *evd, void *user_pointer) {
 			break;
 	}
 }
+
+void media_stream_print_summary(MediaStream *ms){
+	ms_message("MediaStream[%p] (%s) with RtpSession[%p] summary:", ms, ms_format_type_to_string(ms->type), ms->sessions.rtp_session);
+	ms_message("send-ssrc = [dec:%u hex:%x]", rtp_session_get_send_ssrc(ms->sessions.rtp_session), rtp_session_get_send_ssrc(ms->sessions.rtp_session));
+	ms_message("recv-ssrc = [dec:%u hex:%x]",rtp_session_get_recv_ssrc(ms->sessions.rtp_session), rtp_session_get_recv_ssrc(ms->sessions.rtp_session));
+	if (ms->ice_check_list != NULL) {
+		ice_check_list_print_route(ms->ice_check_list, "ICE route:");
+		ms->ice_check_list = NULL;
+	}
+	rtp_stats_display(rtp_session_get_stats(ms->sessions.rtp_session),
+		"                     RTP STATISTICS                          ");
+}
+
+uint32_t media_stream_get_send_ssrc(const MediaStream *stream) {
+	return rtp_session_get_send_ssrc(stream->sessions.rtp_session);
+}
+
+uint32_t media_stream_get_recv_ssrc(const MediaStream *stream) {
+	return rtp_session_get_recv_ssrc(stream->sessions.rtp_session);
+}
