@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -35,6 +36,10 @@
 #define INT32_MAX 017777777777
 #endif
 
+#ifdef HAVE_LIBYUV_H
+#include "libyuv.h"
+#endif
+
 bool_t ms_rect_equal(const MSRect *r1, const MSRect *r2) {
 	return r1->x == r2->x && r1->y == r2->y && r1->w == r2->w && r1->h == r2->h;
 }
@@ -453,6 +458,65 @@ void ms_rgb_to_yuv(const uint8_t rgb[3], uint8_t yuv[3]){
 	yuv[2]=(uint8_t)(0.439*rgb[0] - 0.368*rgb[1] - 0.071*rgb[2] + 128);
 }
 
+#ifdef HAVE_LIBYUV_H
+struct _MSYuvScalerContext{
+	MSVideoSize source;
+	MSVideoSize target;
+	MSPixFmt src_fmt;
+};
+
+typedef struct _MSYuvScalerContext MSYuvScalerContext;
+
+static MSScalerContext *yuv_create_scale_context(int src_w, int src_h, MSPixFmt src_fmt,
+										  int dst_w, int dst_h, MSPixFmt dst_fmt, int flags){
+	MSYuvScalerContext *ctx=ms_new0(MSYuvScalerContext,1);
+	ctx->source.width = src_w;
+	ctx->source.height = src_h;
+	ctx->target.width = dst_w;
+	ctx->target.height = dst_h;
+	ctx->src_fmt = src_fmt;
+	return (MSScalerContext*)ctx;
+}
+
+static int yuv_scale(MSScalerContext *ctx, uint8_t *src[], int src_strides[], uint8_t *dst[], int dst_strides[]){
+	MSYuvScalerContext *fctx=(MSYuvScalerContext*)ctx;
+	int err=-1;
+	
+	switch (fctx->src_fmt) {
+		case MS_YUV420P:
+			err=I420Scale(src[0], src_strides[0], src[1], src_strides[1], src[2], src_strides[2], fctx->source.width, fctx->source.height, dst[0], dst_strides[0], dst[1], dst_strides[1], dst[2], dst_strides[2], fctx->target.width, fctx->target.height, kFilterBilinear);
+			break;
+		case MS_YUY2: case MS_YUYV:
+			err=YUY2ToI420(src[0], src_strides[0], dst[0], dst_strides[0],dst[1], dst_strides[1], dst[2], dst_strides[2], fctx->target.width, fctx->target.height);
+			break;
+		break;
+		case MS_UYVY:
+			err=UYVYToI420(src[0], src_strides[0], dst[0], dst_strides[0],dst[1], dst_strides[1], dst[2], dst_strides[2], fctx->target.width, fctx->target.height);
+		break;
+		case MS_RGB24: /*BI_RGB on windows*/
+			err=RGB24ToJ420(src[0], src_strides[0], dst[0], dst_strides[0],dst[1], dst_strides[1], dst[2], dst_strides[2], fctx->target.width, fctx->target.height);
+		break;
+		default:
+			ms_warning("yuv_scale: unsupported format %s", ms_pix_fmt_to_string(fctx->src_fmt));
+		break;
+	}
+	if(err != 0)// err can be negative or positive in case of error.
+		return -1;
+	else
+		return 0;
+}
+
+static void yuv_free(MSScalerContext *ctx){
+	ms_free(ctx);
+}
+
+MSScalerDesc yuv_scaler={
+	yuv_create_scale_context,
+	yuv_scale,
+	yuv_free
+};
+#endif
+
 #if !defined(NO_FFMPEG)
 
 
@@ -684,6 +748,8 @@ MSScalerDesc * ms_video_get_scaler_impl(void){
 	if (android_getCpuFamily() == ANDROID_CPU_FAMILY_ARM && (android_getCpuFeatures() & ANDROID_CPU_ARM_FEATURE_NEON) != 0){
 		scaler_impl = &ms_android_scaler;
 	}
+#elif defined(HAVE_LIBYUV_H)
+	scaler_impl=&yuv_scaler;
 #elif !defined(NO_FFMPEG)
 	scaler_impl=&ffmpeg_scaler;
 #endif
@@ -1035,7 +1101,7 @@ MSVideoConfiguration ms_video_find_best_configuration_for_size(const MSVideoConf
 
 	/* search for configuration that is first nearest to target video size, then second has the greater fps,
 	 * but any case making sure the the cpu count is sufficient*/
-	while(TRUE) {
+	while(vconf_it) {
 		int pixels=vconf_it->vsize.width*vconf_it->vsize.height;
 		int score=abs(pixels-ref_pixels);
 		if (cpu_count>=vconf_it->mincpu){
