@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -655,14 +656,14 @@ static void stateful_analyzer_update(MSQosAnalyzer *objbase){
 		switch (obj->burst_state){
 		case MSStatefulQosAnalyzerBurstEnable:{
 			obj->burst_state=MSStatefulQosAnalyzerBurstInProgress;
-			ortp_gettimeofday(&obj->start_time, NULL);
+			bctbx_gettimeofday(&obj->start_time, NULL);
 			rtp_session_set_duplication_ratio(obj->session, (float)obj->burst_ratio);
 			BCTBX_NO_BREAK; /*intentionally no break*/
 		} case MSStatefulQosAnalyzerBurstInProgress: {
 			struct timeval now;
 			float elapsed;
 
-			ortp_gettimeofday(&now,NULL);
+			bctbx_gettimeofday(&now,NULL);
 			elapsed=((now.tv_sec-obj->start_time.tv_sec)*1000.0f) +  ((now.tv_usec-obj->start_time.tv_usec)/1000.0f);
 
 			if (elapsed > obj->burst_duration_ms){
