@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of mediastreamer2.
+ * This file is part of mediastreamer2 
+ * (see https://gitlab.linphone.org/BC/public/mediastreamer2).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -162,13 +163,8 @@ void text_stream_stop(TextStream *stream) {
 			ms_ticker_detach(stream->ms.sessions.ticker, stream->rttsource);
 			ms_ticker_detach(stream->ms.sessions.ticker, stream->ms.rtprecv);
 			
-			if (stream->ms.ice_check_list != NULL) {
-				ice_check_list_print_route(stream->ms.ice_check_list, "Text session's route");
-				stream->ms.ice_check_list = NULL;
-			}
-			
-			rtp_stats_display(rtp_session_get_stats(stream->ms.sessions.rtp_session),
-					"             TEXT SESSION'S RTP STATISTICS                ");
+			ms_message("Stopping TextStream");
+			media_stream_print_summary(&stream->ms);
 			
 			ms_connection_helper_start(&h);
 			ms_connection_helper_unlink(&h, stream->rttsource, -1, 0);
